@@ -4,7 +4,7 @@
 /*                                                                         */
 /*    Amiga-specific FreeType low-level system interface (body).           */
 /*                                                                         */
-/*  Copyright 1996-2001, 2002, 2005, 2006 by                               */
+/*  Copyright 1996-2015 by                                                 */
 /*  David Turner, Robert Wilhelm, Werner Lemberg and Detlef Wrkner.       */
 /*                                                                         */
 /*  This file is part of the FreeType project, and may only be used,       */
@@ -33,7 +33,7 @@
   /* malloc(), realloc(), and free().                                      */
   /*                                                                       */
   /* Those C library functions are often not thread-safe or cant be        */
-  /* used in a shared Amiga library. If thats not a problem for you,       */
+  /* used in a shared Amiga library. If that's not a problem for you,       */
   /* you can of course use the default ftsystem.c with C library calls     */
   /* instead.                                                              */
   /*                                                                       */
@@ -386,7 +386,7 @@ Free_VecPooled( APTR  poolHeader,
 
 
     if ( !stream )
-      return FT_Err_Invalid_Stream_Handle;
+      return FT_THROW( Invalid_Stream_Handle );
 
 #ifdef __amigaos4__
     sysfile = AllocMem ( sizeof (struct SysFile ), MEMF_SHARED );
@@ -398,7 +398,7 @@ Free_VecPooled( APTR  poolHeader,
       FT_ERROR(( "FT_Stream_Open:" ));
       FT_ERROR(( " could not open `%s'\n", filepathname ));
 
-      return FT_Err_Cannot_Open_Resource;
+      return FT_THROW( Cannot_Open_Resource );
     }
     sysfile->file = Open( (STRPTR)filepathname, MODE_OLDFILE );
     if ( !sysfile->file )
@@ -407,7 +407,7 @@ Free_VecPooled( APTR  poolHeader,
       FT_ERROR(( "FT_Stream_Open:" ));
       FT_ERROR(( " could not open `%s'\n", filepathname ));
 
-      return FT_Err_Cannot_Open_Resource;
+      return FT_THROW( Cannot_Open_Resource );
     }
 
     fib = AllocDosObject( DOS_FIB, NULL );
@@ -418,7 +418,7 @@ Free_VecPooled( APTR  poolHeader,
       FT_ERROR(( "FT_Stream_Open:" ));
       FT_ERROR(( " could not open `%s'\n", filepathname ));
 
-      return FT_Err_Cannot_Open_Resource;
+      return FT_THROW( Cannot_Open_Resource );
     }
     if ( !( ExamineFH( sysfile->file, fib ) ) )
     {
@@ -428,7 +428,7 @@ Free_VecPooled( APTR  poolHeader,
       FT_ERROR(( "FT_Stream_Open:" ));
       FT_ERROR(( " could not open `%s'\n", filepathname ));
 
-      return FT_Err_Cannot_Open_Resource;
+      return FT_THROW( Cannot_Open_Resource );
     }
     stream->size = fib->fib_Size;
     FreeDosObject( DOS_FIB, fib );
@@ -442,6 +442,14 @@ Free_VecPooled( APTR  poolHeader,
     stream->read  = ft_amiga_stream_io;
     stream->close = ft_amiga_stream_close;
 
+    if ( !stream->size )
+    {
+      ft_amiga_stream_close( stream );
+      FT_ERROR(( "FT_Stream_Open:" ));
+      FT_ERROR(( " opened `%s' but zero-sized\n", filepathname ));
+      return FT_THROW( Cannot_Open_Stream );
+    }
+
     FT_TRACE1(( "FT_Stream_Open:" ));
     FT_TRACE1(( " opened `%s' (%ld bytes) successfully\n",
                 filepathname, stream->size ));
@@ -457,7 +465,7 @@ Free_VecPooled( APTR  poolHeader,
 
   extern void
   ft_mem_debug_done( FT_Memory  memory );
-  
+
 #endif
 
 
@@ -493,7 +501,7 @@ Free_VecPooled( APTR  poolHeader,
         memory->free    = ft_free;
 #ifdef FT_DEBUG_MEMORY
         ft_mem_debug_init( memory );
-#endif    
+#endif
       }
     }
 
@@ -508,7 +516,7 @@ Free_VecPooled( APTR  poolHeader,
   {
 #ifdef FT_DEBUG_MEMORY
     ft_mem_debug_done( memory );
-#endif  
+#endif
 
     DeletePool( memory->user );
     FreeVec( memory );
