@@ -7,7 +7,7 @@ gblender_set_gamma_table( double           gamma_value,
                           unsigned short*  gamma_ramp,
                           unsigned char*   gamma_ramp_inv )
 {
-  int    gmax      = (256 << GBLENDER_GAMMA_SHIFT)-1;
+  int  gmax = (256 << GBLENDER_GAMMA_SHIFT)-1;
 
   if ( gamma_value <= 0 )  /* special case for sRGB */
   {
@@ -40,8 +40,8 @@ gblender_set_gamma_table( double           gamma_value,
   }
   else
   {
-    int    ii;
-    double gamma_inv = 1.0f / gamma_value;
+    int     ii;
+    double  gamma_inv = 1.0f / gamma_value;
 
     /* voltage to linear */
     for ( ii = 0; ii < 256; ii++ )
@@ -71,16 +71,16 @@ gblender_clear( GBlender  blender )
     for ( nn = 0; nn < GBLENDER_KEY_COUNT; nn++ )
       chan_keys[nn].index = -1;
 
-    blender->cache_r_back  = -1;
-    blender->cache_r_fore  = -1;
+    blender->cache_r_back  = ~0U;
+    blender->cache_r_fore  = ~0U;
     blender->cache_r_cells = NULL;
 
-    blender->cache_g_back  = -1;
-    blender->cache_g_fore  = -1;
+    blender->cache_g_back  = ~0U;
+    blender->cache_g_fore  = ~0U;
     blender->cache_g_cells = NULL;
 
-    blender->cache_b_back  = -1;
-    blender->cache_b_fore  = -1;
+    blender->cache_b_back  = ~0U;
+    blender->cache_b_fore  = ~0U;
     blender->cache_g_cells = NULL;
   }
   else
@@ -88,8 +88,8 @@ gblender_clear( GBlender  blender )
     for ( nn = 0; nn < GBLENDER_KEY_COUNT; nn++ )
       keys[nn].cells = NULL;
 
-    blender->cache_back  = -1;
-    blender->cache_fore  = -1;
+    blender->cache_back  = ~0U;
+    blender->cache_fore  = ~0U;
     blender->cache_cells = NULL;
   }
 }
@@ -172,13 +172,13 @@ gblender_reset_key( GBlender     blender,
   GBlenderPixel  back = key->background;
   GBlenderPixel  fore = key->foreground;
   GBlenderCell*  gr   = key->cells;
-  int            nn;
+  unsigned int   nn;
   int            gmax = (256 << GBLENDER_GAMMA_SHIFT)-1;
 
   const unsigned char*   gamma_ramp_inv = blender->gamma_ramp_inv;
   const unsigned short*  gamma_ramp     = blender->gamma_ramp;
 
-  int  r1,g1,b1,r2,g2,b2;
+  unsigned int  r1,g1,b1,r2,g2,b2;
 
   r1 = ( back >> 16 ) & 255;
   g1 = ( back >> 8 )  & 255;
@@ -208,9 +208,9 @@ gblender_reset_key( GBlender     blender,
 
   for ( nn = 1; nn < GBLENDER_SHADE_COUNT; nn++ )
   {
-    int  bits = 8;
-    int  a    = 0;
-    int  r, g, b;
+    unsigned int  bits = 8;
+    unsigned int  a    = 0;
+    int           r, g, b;
 
     while ( bits >= GBLENDER_SHADE_BITS )
     {
@@ -223,17 +223,17 @@ gblender_reset_key( GBlender     blender,
       a   += (nn >> bits);
     }
 
-    r = ((r2-r1)*a + 128);
-    g = ((g2-g1)*a + 128);
-    b = ((g2-g1)*a + 128);
+    r = (int)((r2-r1)*a + 128);
+    g = (int)((g2-g1)*a + 128);
+    b = (int)((g2-g1)*a + 128);
 
     r = (r + (r >> 8)) >> 8;
     g = (g + (g >> 8)) >> 8;
     b = (b + (b >> 8)) >> 8;
 
-    r += r1;
-    g += g1;
-    b += b1;
+    r += (int)r1;
+    g += (int)g1;
+    b += (int)b1;
 
 #if 0
     r = ( r | -(r >> 8) ) & 255;
@@ -255,9 +255,9 @@ gblender_reset_key( GBlender     blender,
    gr[2] = (unsigned char)b;
    gr   += 3;
 #else
-    gr[0] = (( r & 255 ) << 16) |
-            (( g & 255 ) << 8 ) |
-            (( b & 255 )      ) ;
+    gr[0] = (( (unsigned int)r & 255 ) << 16) |
+            (( (unsigned int)g & 255 ) << 8 ) |
+            (( (unsigned int)b & 255 )      ) ;
     gr ++;
 #endif
   }
@@ -332,21 +332,21 @@ static void
 gblender_reset_channel_key( GBlender         blender,
                             GBlenderChanKey  key )
 {
-  int            back = key->backfore & 255;
-  int            fore = (key->backfore >> 8) & 255;
-  unsigned char* gr   = (unsigned char*)blender->cells + key->index;
-  int            nn;
+  unsigned int    back = key->backfore & 255;
+  unsigned int    fore = (key->backfore >> 8) & 255;
+  unsigned char*  gr   = (unsigned char*)blender->cells + key->index;
+  unsigned int    nn;
 
   const unsigned char*   gamma_ramp_inv = blender->gamma_ramp_inv;
   const unsigned short*  gamma_ramp     = blender->gamma_ramp;
 
-  int  r1,r2;
-  int  gmax = (256 << GBLENDER_GAMMA_SHIFT)-1;
+  unsigned int  r1,r2;
+  int           gmax = (256 << GBLENDER_GAMMA_SHIFT)-1;
 
   r1    = back;
   r2    = fore;
 
-  gr[0] = r1;
+  gr[0] = (unsigned char)r1;
   gr++;
 
 
@@ -355,8 +355,8 @@ gblender_reset_channel_key( GBlender         blender,
 
   for ( nn = 1; nn < GBLENDER_SHADE_COUNT; nn++ )
   {
-    int  bits = 8;
-    int  a    = 0;
+    unsigned int  bits = 8;
+    unsigned int  a    = 0;
     int  r;
 
     while ( bits >= GBLENDER_SHADE_BITS )
@@ -367,9 +367,9 @@ gblender_reset_channel_key( GBlender         blender,
     if ( bits > 0 )
       a += (nn >> (GBLENDER_SHADE_BITS-bits));
 
-    r = ((r2-r1)*a + 128);
+    r = (int)((r2-r1)*a + 128);
     r = (r + (r >> 8)) >> 8;
-    r += r1;
+    r += (int)r1;
     if ( r < 0 ) r = 0; else if ( r > gmax ) r = gmax;
     r  = gamma_ramp_inv[r];
 
@@ -380,9 +380,9 @@ gblender_reset_channel_key( GBlender         blender,
 
 
 GBLENDER_APIDEF( unsigned char* )
-gblender_lookup_channel( GBlender       blender,
-                         int            background,
-                         int            foreground )
+gblender_lookup_channel( GBlender      blender,
+                         unsigned int  background,
+                         unsigned int  foreground )
 {
   int              idx, idx0;
   unsigned short   backfore = (unsigned short)((foreground << 8) | background);
