@@ -4,7 +4,7 @@
  *
  *    Graphics Subsystem interface
  *
- *  Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005
+ *  Copyright 1999-2007, 2013
  *     - The FreeType Development Team - www.freetype.org
  *
  ***************************************************************************/
@@ -46,6 +46,7 @@
     gr_pixel_mode_lcdv,        /* vertical RGB-decimated           */
     gr_pixel_mode_lcd2,        /* horizontal BGR-decimated         */
     gr_pixel_mode_lcdv2,       /* vertical BGR-decimated           */
+    gr_pixel_mode_bgra,        /* premultiplied BGRA colors        */
 
     gr_pixel_mode_max          /* don't remove */
 
@@ -190,6 +191,14 @@
                        grColor    color );
 
 
+  /* values must be in 0..255 range */
+  extern grColor
+  grFindColor( grBitmap*  target,
+               int        red,
+               int        green,
+               int        blue,
+               int        alpha );
+
 #if 0
 
  /**********************************************************************
@@ -297,6 +306,27 @@
   extern  void  grDoneBitmap( grBitmap*  bit );
 
 
+  extern void
+  grFillHLine( grBitmap*  target,
+               int        x,
+               int        y,
+               int        width,
+               grColor    color );
+
+  extern void
+  grFillVLine( grBitmap*  target,
+               int        x,
+               int        y,
+               int        height,
+               grColor    color );
+
+  extern void
+  grFillRect( grBitmap*   target,
+              int         x,
+              int         y,
+              int         width,
+              int         height,
+              grColor     color );
 
 
  /*************************************************************************/
@@ -367,7 +397,7 @@
   *    the device chain returned by this function. For example, if an
   *    X11 device was compiled in the library, it will be part of
   *    the returned device chain only if a connection to the display
-  *    could be establisged
+  *    could be established
   *
   *    If no driver could be initialised, this function returns NULL.
   *
