@@ -2,11 +2,11 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2005, 2006, 2007, 2008, 2009 by                               */
+/*  Copyright 2005-2009, 2011-2013 by                                       */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
-/*  ftcommon.c - common routines for the FreeType demo programs.            */
+/*  ftcommon.c - common routines for the graphic FreeType demo programs.    */
 /*                                                                          */
 /****************************************************************************/
 
@@ -18,6 +18,7 @@
 #include FT_CACHE_MANAGER_H
 
 #include FT_BITMAP_H
+#include FT_FONT_FORMATS_H
 
 #include "common.h"
 #include "ftcommon.h"
@@ -33,7 +34,7 @@
 #endif
 
 
-  FT_Error   error;
+  FT_Error  error;
 
 
 #undef  NODEBUG
@@ -41,7 +42,8 @@
 #ifndef NODEBUG
 
   void
-  LogMessage( const char*  fmt, ... )
+  LogMessage( const char*  fmt,
+              ... )
   {
     va_list  ap;
 
@@ -72,19 +74,17 @@
   /*************************************************************************/
 
 
-#define DIM_X      500
-#define DIM_Y      400
-
-
   FTDemo_Display*
-  FTDemo_Display_New( grPixelMode  mode )
+  FTDemo_Display_New( grPixelMode  mode,
+                      int          width,
+                      int          height )
   {
     FTDemo_Display*  display;
     grSurface*       surface;
     grBitmap         bit;
 
 
-    display = (FTDemo_Display *)malloc( sizeof( FTDemo_Display ) );
+    display = (FTDemo_Display *)malloc( sizeof ( FTDemo_Display ) );
     if ( !display )
       return NULL;
 
@@ -95,8 +95,8 @@
     grInitDevices();
 
     bit.mode  = mode;
-    bit.width = DIM_X;
-    bit.rows  = DIM_Y;
+    bit.width = width;
+    bit.rows  = height;
     bit.grays = 256;
 
     surface = grNewSurface( 0, &bit );
@@ -112,8 +112,12 @@
 
     grSetGlyphGamma( 1.0 );
 
-    memset( &display->fore_color, 0, sizeof( grColor ) );
-    memset( &display->back_color, 0xff, sizeof( grColor ) );
+    display->fore_color = grFindColor( display->bitmap,
+                                       0x00, 0x00, 0x00, 0xff );
+    display->back_color = grFindColor( display->bitmap,
+                                       0xff, 0xff, 0xff, 0xff );
+    display->warn_color = grFindColor( display->bitmap,
+                                       0xff, 0x00, 0x00, 0xff );
 
     return display;
   }
@@ -145,7 +149,9 @@
       pitch = -pitch;
 
     if ( bit->mode == gr_pixel_mode_gray )
-      memset( bit->buffer, display->back_color.value, pitch * bit->rows );
+      memset( bit->buffer,
+              display->back_color.value,
+              (unsigned int)( pitch * bit->rows ) );
     else
     {
       unsigned char*  p = bit->buffer;
@@ -178,19 +184,6 @@
 #define TRUNC( x )  (   (x) >> 6 )
 
 
-
-  static
-  const char*  file_suffixes[] =
-  {
-    ".ttf",
-    ".ttc",
-    ".otf",
-    ".pfa",
-    ".pfb",
-    0
-  };
-
-
   /*************************************************************************/
   /*                                                                       */
   /* The face requester is a function provided by the client application   */
@@ -209,10 +202,11 @@
 
     FT_UNUSED( request_data );
 
+
     if ( font->file_address != NULL )
       error = FT_New_Memory_Face( lib,
                                   (const FT_Byte*)font->file_address,
-                                  font->file_size,
+                                  (FT_Long)font->file_size,
                                   font->face_index,
                                   aface );
     else
@@ -222,23 +216,33 @@
                            aface );
     if ( !error )
     {
-      char*  suffix;
-      char   orig[4];
+      const char*  format = FT_Get_Font_Format( *aface );
 
 
-      suffix = strrchr( font->filepathname, '.' );
-      if ( suffix && ( strcasecmp( suffix, ".pfa" ) == 0 ||
-                       strcasecmp( suffix, ".pfb" ) == 0 ) )
+      if ( !strcmp( format, "Type 1" ) )
       {
-        suffix++;
+        char   orig[5];
+        char*  suffix        = (char*)strrchr( font->filepathname, '.' );
+        int    has_extension = suffix                                &&
+                               ( strcasecmp( suffix, ".pfa" ) == 0 ||
+                                 strcasecmp( suffix, ".pfb" ) == 0 );
 
-        memcpy( orig, suffix, 4 );
-        memcpy( suffix, "afm", 4 );
-        FT_Attach_File( *aface, font->filepathname );
 
-        memcpy( suffix, "pfm", 4 );
-        FT_Attach_File( *aface, font->filepathname );
-        memcpy( suffix, orig, 4 );
+        if ( has_extension )
+          memcpy( orig, suffix, 5 );
+        else
+          /* we have already allocated four more bytes */
+          suffix = (char*)font->filepathname + strlen( font->filepathname );
+
+        memcpy( suffix, ".afm", 5 );
+        if ( FT_Attach_File( *aface, font->filepathname ) )
+        {
+          memcpy( suffix, ".pfm", 5 );
+          FT_Attach_File( *aface, font->filepathname );
+        }
+
+        if ( has_extension )
+          memcpy( suffix, orig, 5 );
       }
 
       if ( (*aface)->charmaps )
@@ -250,16 +254,16 @@
 
 
   FTDemo_Handle*
-  FTDemo_New( FT_Encoding encoding )
+  FTDemo_New( void )
   {
     FTDemo_Handle*  handle;
 
 
-    handle = (FTDemo_Handle *)malloc( sizeof( FTDemo_Handle ) );
+    handle = (FTDemo_Handle *)malloc( sizeof ( FTDemo_Handle ) );
     if ( !handle )
       return NULL;
 
-    memset( handle, 0, sizeof( FTDemo_Handle ) );
+    memset( handle, 0, sizeof ( FTDemo_Handle ) );
 
     error = FT_Init_FreeType( &handle->library );
     if ( error )
@@ -282,22 +286,23 @@
     if ( error )
       PanicZ( "could not initialize charmap cache" );
 
-
     FT_Bitmap_New( &handle->bitmap );
 
-    handle->encoding = encoding;
+    FT_Stroker_New( handle->library, &handle->stroker );
+
+    handle->encoding = FT_ENCODING_NONE;
 
     handle->hinted    = 1;
     handle->antialias = 1;
     handle->use_sbits = 1;
-    handle->low_prec  = 0;
     handle->autohint  = 0;
     handle->lcd_mode  = 0;
+    handle->color     = 1;
 
-    handle->use_sbits_cache  = 1;
+    handle->use_sbits_cache = 1;
 
     /* string_init */
-    memset( handle->string, 0, sizeof( TGlyph ) * MAX_GLYPHS );
+    memset( handle->string, 0, sizeof ( TGlyph ) * MAX_GLYPHS );
     handle->string_length = 0;
     handle->string_reload = 1;
 
@@ -335,6 +340,7 @@
         FT_Done_Glyph( glyph->image );
     }
 
+    FT_Stroker_Done( handle->stroker );
     FT_Bitmap_Done( handle->library, &handle->bitmap );
     FTC_Manager_Done( handle->cache_manager );
     FT_Done_FreeType( handle->library );
@@ -345,11 +351,13 @@
 
   FT_Error
   FTDemo_Install_Font( FTDemo_Handle*  handle,
-                       const char*     filepath )
+                       const char*     filepath,
+                       FT_Bool         outline_only )
   {
-    static char  filename[1024 + 5];
-    int          i, len, num_faces;
-    FT_Face      face;
+    static char   filename[1024 + 5];
+    long          i, num_faces;
+    unsigned int  len;
+    FT_Face       face;
 
 
     len = strlen( filepath );
@@ -359,161 +367,151 @@
     strncpy( filename, filepath, len );
     filename[len] = 0;
 
-    error = FT_New_Face( handle->library, filename, 0, &face );
+    /* We use a conservative approach here, at the cost of calling     */
+    /* `FT_New_Face' quite often.  The idea is that our demo programs  */
+    /* should be able to try all faces and named instances of a font,  */
+    /* expecting that some faces don't work for various reasons, e.g., */
+    /* a broken subfont, or an unsupported NFNT bitmap font in a Mac   */
+    /* dfont resource that holds more than a single font.              */
 
-#ifndef macintosh
-    /* could not open the file directly; we will now try various */
-    /* suffixes like `.ttf' or `.pfb'                            */
+    error = FT_New_Face( handle->library, filename, -1, &face );
     if ( error )
-    {
-      const char**  suffix;
-      char*         p;
-      int           found = 0;
-
-      suffix = file_suffixes;
-      p      = filename + len - 1;
-
-      while ( p >= filename && *p != '\\' && *p != '/' )
-      {
-        if ( *p == '.' )
-          break;
-
-        p--;
-      }
-
-      /* no suffix found */
-      if ( p < filename || *p != '.' )
-        p = filename + len;
-
-      for ( suffix = file_suffixes; suffix[0]; suffix++ )
-      {
-        /* try with current suffix */
-        strcpy( p, suffix[0] );
-
-        error = FT_New_Face( handle->library, filename, 0, &face );
-        if ( !error )
-        {
-          found = 1;
-
-          break;
-        }
-      }
-
-      /* really couldn't open this file */
-      if ( !found )
-        return error;
-    }
-#endif /* !macintosh */
-
-    /* allocate new font object */
+      return error;
     num_faces = face->num_faces;
+    FT_Done_Face( face );
+
+    /* allocate new font object(s) */
     for ( i = 0; i < num_faces; i++ )
     {
       PFont  font;
+      long   j, instance_count;
 
 
-      if ( i > 0 )
+      error = FT_New_Face( handle->library, filename, -( i + 1 ), &face );
+      if ( error )
+        continue;
+      instance_count = face->style_flags >> 16;
+      FT_Done_Face( face );
+
+      /* load face with and without named instances */
+      for ( j = 0; j < instance_count + 1; j++ )
       {
-        error = FT_New_Face( handle->library, filename, i, &face );
+        error = FT_New_Face( handle->library,
+                             filename,
+                             ( j << 16 ) + i,
+                             &face );
         if ( error )
           continue;
-      }
 
-      if ( handle->encoding != FT_ENCODING_NONE )
-      {
-        error = FT_Select_Charmap( face, handle->encoding );
-        if ( error )
+        if ( outline_only && !FT_IS_SCALABLE( face ) )
         {
           FT_Done_Face( face );
-          return error;
+          continue;
         }
-      }
 
-      font = (PFont)malloc( sizeof ( *font ) );
-
-      font->filepathname = (char*)malloc( strlen( filename ) + 1 );
-      strcpy( (char*)font->filepathname, filename );
-
-      font->face_index = i;
-      font->cmap_index = face->charmap ? FT_Get_Charmap_Index( face->charmap )
-                                       : 0;
-
-      if ( handle->preload )
-      {
-        FILE*   file = fopen( filename, "rb" );
-        size_t  file_size;
-
-        if ( file == NULL )  /* shouldn't happen */
+        if ( handle->encoding != FT_ENCODING_NONE )
         {
-          free( font );
-          return FT_Err_Invalid_Argument;
+          error = FT_Select_Charmap( face, handle->encoding );
+          if ( error )
+          {
+            FT_Done_Face( face );
+            continue;
+          }
         }
 
-        fseek( file, 0, SEEK_END );
-        file_size = ftell( file );
-        fseek( file, 0, SEEK_SET );
+        font = (PFont)malloc( sizeof ( *font ) );
 
-        font->file_address = malloc( file_size );
-        fread( font->file_address, 1, file_size, file );
+        /* We allocate four more bytes since we want to attach an AFM */
+        /* or PFM file for Type 1 fonts (if available).  Such fonts   */
+        /* always have the extension `.afm' or `.pfm'.                */
+        font->filepathname = (char*)malloc( strlen( filename ) + 4 + 1 );
+        strcpy( (char*)font->filepathname, filename );
 
-        font->file_size    = file_size;
+        font->face_index = ( j << 16 ) + i;
+        font->cmap_index = face->charmap ? FT_Get_Charmap_Index( face->charmap )
+                                         : 0;
 
-        fclose( file );
+        if ( handle->preload )
+        {
+          FILE*  file = fopen( filename, "rb" );
+          int    file_size;
+
+
+          if ( file == NULL )  /* shouldn't happen */
+          {
+            free( font );
+            return FT_Err_Invalid_Argument;
+          }
+
+          fseek( file, 0, SEEK_END );
+          file_size = ftell( file );
+          fseek( file, 0, SEEK_SET );
+
+          if ( file_size <= 0 )
+            return FT_Err_Invalid_Stream_Operation;
+
+          font->file_address = malloc( (size_t)file_size );
+          fread( font->file_address, 1, (size_t)file_size, file );
+
+          font->file_size = (size_t)file_size;
+
+          fclose( file );
+        }
+        else
+        {
+          font->file_address = NULL;
+          font->file_size    = 0;
+        }
+
+        switch ( handle->encoding )
+        {
+        case FT_ENCODING_NONE:
+          font->num_indices = face->num_glyphs;
+          break;
+
+        case FT_ENCODING_UNICODE:
+          font->num_indices = 0x110000L;
+          break;
+
+        case FT_ENCODING_ADOBE_LATIN_1:
+        case FT_ENCODING_ADOBE_STANDARD:
+        case FT_ENCODING_ADOBE_EXPERT:
+        case FT_ENCODING_ADOBE_CUSTOM:
+        case FT_ENCODING_APPLE_ROMAN:
+          font->num_indices = 0x100L;
+          break;
+
+          /* some fonts use range 0x00-0x100, others have 0xF000-0xF0FF */
+        case FT_ENCODING_MS_SYMBOL:
+          font->num_indices = 0x10000L;
+
+        default:
+          font->num_indices = 0x10000L;
+        }
+
+        FT_Done_Face( face );
+        face = NULL;
+
+        if ( handle->max_fonts == 0 )
+        {
+          handle->max_fonts = 16;
+          handle->fonts     = (PFont*)calloc( (size_t)handle->max_fonts,
+                                              sizeof ( PFont ) );
+        }
+        else if ( handle->num_fonts >= handle->max_fonts )
+        {
+          handle->max_fonts *= 2;
+          handle->fonts      = (PFont*)realloc( handle->fonts,
+                                                (size_t)handle->max_fonts *
+                                                  sizeof ( PFont ) );
+
+          memset( &handle->fonts[handle->num_fonts], 0,
+                  (size_t)( handle->max_fonts - handle->num_fonts ) *
+                    sizeof ( PFont ) );
+        }
+
+        handle->fonts[handle->num_fonts++] = font;
       }
-      else
-      {
-        font->file_address = NULL;
-        font->file_size    = 0;
-      }
-
-      switch ( handle->encoding )
-      {
-      case FT_ENCODING_NONE:
-        font->num_indices = face->num_glyphs;
-        break;
-
-      case FT_ENCODING_UNICODE:
-        font->num_indices = 0x110000L;
-        break;
-
-      case FT_ENCODING_ADOBE_LATIN_1:
-      case FT_ENCODING_ADOBE_STANDARD:
-      case FT_ENCODING_ADOBE_EXPERT:
-      case FT_ENCODING_ADOBE_CUSTOM:
-      case FT_ENCODING_APPLE_ROMAN:
-        font->num_indices = 0x100L;
-        break;
-
-        /* some fonts use range 0x00-0x100, others have 0xF000-0xF0FF */
-      case FT_ENCODING_MS_SYMBOL:
-        font->num_indices = 0x10000L;
-
-      default:
-        font->num_indices = 0x10000L;
-      }
-
-      FT_Done_Face( face );
-      face = NULL;
-
-      if ( handle->max_fonts == 0 )
-      {
-        handle->max_fonts = 16;
-        handle->fonts     = (PFont*)calloc( handle->max_fonts,
-                                            sizeof ( PFont ) );
-      }
-      else if ( handle->num_fonts >= handle->max_fonts )
-      {
-        handle->max_fonts *= 2;
-        handle->fonts      = (PFont*)realloc( handle->fonts,
-                                              handle->max_fonts *
-                                                sizeof ( PFont ) );
-
-        memset( &handle->fonts[handle->num_fonts], 0,
-                ( handle->max_fonts - handle->num_fonts ) *
-                  sizeof ( PFont ) );
-      }
-
-      handle->fonts[handle->num_fonts++] = font;
     }
 
     return FT_Err_Ok;
@@ -524,7 +522,7 @@
   FTDemo_Set_Current_Font( FTDemo_Handle*  handle,
                            PFont           font )
   {
-    handle->current_font = font;
+    handle->current_font   = font;
     handle->scaler.face_id = (FTC_FaceID)font;
 
     handle->string_reload = 1;
@@ -538,32 +536,35 @@
     if ( pixel_size > 0xFFFF )
       pixel_size = 0xFFFF;
 
-    handle->scaler.width  = (FT_UInt) pixel_size;
-    handle->scaler.height = (FT_UInt) pixel_size;
-    handle->scaler.pixel  = 1;
+    handle->scaler.width  = (FT_UInt)pixel_size;
+    handle->scaler.height = (FT_UInt)pixel_size;
+    handle->scaler.pixel  = 1;                  /* activate integer format */
     handle->scaler.x_res  = 0;
     handle->scaler.y_res  = 0;
 
     handle->string_reload = 1;
   }
 
+
   void
   FTDemo_Set_Current_Charsize( FTDemo_Handle*  handle,
                                int             char_size,
                                int             resolution )
   {
-      if ( char_size > 0xFFFFF )
-          char_size = 0xFFFFF;
+    /* in 26.6 format, corresponding to (almost) 0x4000ppem */
+    if ( char_size > 0xFFFFF )
+      char_size = 0xFFFFF;
 
-      handle->scaler.width  = (FT_UInt) char_size;
-      handle->scaler.height = (FT_UInt) char_size;
-      handle->scaler.pixel  = 0;
-      handle->scaler.x_res  = (FT_UInt) resolution;
-      handle->scaler.y_res  = (FT_UInt) resolution;
+    handle->scaler.width  = (FT_UInt)char_size;
+    handle->scaler.height = (FT_UInt)char_size;
+    handle->scaler.pixel  = 0;                     /* activate 26.6 format */
+    handle->scaler.x_res  = (FT_UInt)resolution;
+    handle->scaler.y_res  = (FT_UInt)resolution;
 
-      handle->string_reload = 1;
+    handle->string_reload = 1;
   }
 
+
   void
   FTDemo_Set_Preload( FTDemo_Handle*  handle,
                       int             preload )
@@ -571,24 +572,15 @@
     handle->preload = !!preload;
   }
 
-  void
-  FTDemo_Set_Current_Pointsize( FTDemo_Handle*  handle,
-                                int             point_size,
-                                int             res )
-  {
-    FTDemo_Set_Current_Size( handle, ( point_size * res + 36 ) / 72 );
-  }
-
 
   void
   FTDemo_Update_Current_Flags( FTDemo_Handle*  handle )
   {
-    FT_UInt32   flags, target;
+    FT_Int32  flags, target;
+
 
     flags = FT_LOAD_DEFAULT;  /* really 0 */
 
-    flags |= FT_LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH;
-
     if ( handle->autohint )
       flags |= FT_LOAD_FORCE_AUTOHINT;
 
@@ -603,22 +595,22 @@
       {
         switch ( handle->lcd_mode )
         {
-          case LCD_MODE_LIGHT:
-            target = FT_LOAD_TARGET_LIGHT;
-            break;
+        case LCD_MODE_LIGHT:
+          target = FT_LOAD_TARGET_LIGHT;
+          break;
 
-          case LCD_MODE_RGB:
-          case LCD_MODE_BGR:
-            target = FT_LOAD_TARGET_LCD;
-            break;
+        case LCD_MODE_RGB:
+        case LCD_MODE_BGR:
+          target = FT_LOAD_TARGET_LCD;
+          break;
 
-          case LCD_MODE_VRGB:
-          case LCD_MODE_VBGR:
-            target = FT_LOAD_TARGET_LCD_V;
-            break;
+        case LCD_MODE_VRGB:
+        case LCD_MODE_VBGR:
+          target = FT_LOAD_TARGET_LCD_V;
+          break;
 
-          default:
-            target = FT_LOAD_TARGET_NORMAL;
+        default:
+          target = FT_LOAD_TARGET_NORMAL;
         }
       }
       else
@@ -627,8 +619,16 @@
       flags |= target;
     }
     else
+    {
       flags |= FT_LOAD_NO_HINTING;
 
+      if ( !handle->antialias )
+        flags |= FT_LOAD_MONOCHROME;
+    }
+
+    if ( handle->color )
+      flags |= FT_LOAD_COLOR;
+
     handle->load_flags    = flags;
     handle->string_reload = 1;
   }
@@ -651,10 +651,12 @@
   FTDemo_Get_Size( FTDemo_Handle*  handle,
                    FT_Size*        asize )
   {
-    FT_Size        size;
+    FT_Size  size;
 
 
-    error = FTC_Manager_LookupSize( handle->cache_manager, &handle->scaler, &size );
+    error = FTC_Manager_LookupSize( handle->cache_manager,
+                                    &handle->scaler,
+                                    &size );
 
     if ( !error )
       *asize = size;
@@ -712,8 +714,8 @@
     bitmap = (FT_BitmapGlyph)glyf;
     source = &bitmap->bitmap;
 
-    target->rows   = source->rows;
-    target->width  = source->width;
+    target->rows   = (int)source->rows;
+    target->width  = (int)source->width;
     target->pitch  = source->pitch;
     target->buffer = source->buffer;
     target->grays  = source->num_grays;
@@ -750,6 +752,11 @@
       target->grays = source->num_grays;
       break;
 
+    case FT_PIXEL_MODE_BGRA:
+      target->mode  = gr_pixel_mode_bgra;
+      target->grays = source->num_grays;
+      break;
+
     default:
       return FT_Err_Invalid_Glyph_Format;
     }
@@ -774,8 +781,7 @@
                           int*            y_advance,
                           FT_Glyph*       aglyf )
   {
-    int  cached_bitmap = 1;
-    int  width, height;
+    unsigned int  width, height;
 
 
     *aglyf     = NULL;
@@ -787,9 +793,10 @@
 
     width  = handle->scaler.width;
     height = handle->scaler.height;
-    if ( handle->use_sbits_cache && !handle->scaler.pixel ) {
-        width  = (( width * handle->scaler.x_res + 36 ) / 72)  >> 6;
-        height = (( height * handle->scaler.y_res + 36 ) / 72) >> 6;
+    if ( handle->use_sbits_cache && !handle->scaler.pixel )
+    {
+      width  = ( ( width * handle->scaler.x_res + 36 ) / 72 )  >> 6;
+      height = ( ( height * handle->scaler.y_res + 36 ) / 72 ) >> 6;
     }
 
     if ( handle->use_sbits_cache && width < 48 && height < 48 )
@@ -800,7 +807,7 @@
 
       error = FTC_SBitCache_LookupScaler( handle->sbits_cache,
                                           &handle->scaler,
-                                          handle->load_flags,
+                                          (FT_ULong)handle->load_flags,
                                           Index,
                                           &sbit,
                                           NULL );
@@ -823,7 +830,6 @@
 
         case FT_PIXEL_MODE_GRAY:
           target->mode  = gr_pixel_mode_gray;
-          target->grays = sbit->max_grays + 1;
           break;
 
         case FT_PIXEL_MODE_GRAY2:
@@ -841,27 +847,26 @@
           target->buffer = handle->bitmap.buffer;
           target->mode   = gr_pixel_mode_gray;
           target->grays  = handle->bitmap.num_grays;
-
-          cached_bitmap = 0;
           break;
 
         case FT_PIXEL_MODE_LCD:
           target->mode  = handle->lcd_mode == 2 ? gr_pixel_mode_lcd
                                                 : gr_pixel_mode_lcd2;
-          target->grays = sbit->max_grays + 1;
           break;
 
         case FT_PIXEL_MODE_LCD_V:
           target->mode  = handle->lcd_mode == 4 ? gr_pixel_mode_lcdv
                                                 : gr_pixel_mode_lcdv2;
-          target->grays = sbit->max_grays + 1;
+          break;
+
+        case FT_PIXEL_MODE_BGRA:
+          target->mode  = gr_pixel_mode_bgra;
           break;
 
         default:
           return FT_Err_Invalid_Glyph_Format;
         }
 
-
         *left      = sbit->left;
         *top       = sbit->top;
         *x_advance = sbit->xadvance;
@@ -874,11 +879,12 @@
     /* otherwise, use an image cache to store glyph outlines, and render */
     /* them on demand. we can thus support very large sizes easily..     */
     {
-      FT_Glyph   glyf;
+      FT_Glyph  glyf;
+
 
       error = FTC_ImageCache_LookupScaler( handle->image_cache,
                                            &handle->scaler,
-                                           handle->load_flags,
+                                           (FT_ULong)handle->load_flags,
                                            Index,
                                            &glyf,
                                            NULL );
@@ -889,27 +895,6 @@
     }
 
   Exit:
-
-#ifdef FT_RGB_FILTER_H
-   /* note that we apply the RGB filter to each cached glyph, which is
-    * a performance killer, but that's better than modifying the cache
-    * at the moment
-    */
-    if ( !error )
-    {
-      if ( target->mode == gr_pixel_mode_lcd  ||
-           target->mode == gr_pixel_mode_lcdv )
-      {
-       /* copy the bitmap before filtering it, we don't want to touch
-        * the content of cache nodes at all
-        */
-        {
-        }
-      }
-    }
-
-#endif /* FT_RGB_FILTER_H */
-
     /* don't accept a `missing' character with zero or negative width */
     if ( Index == 0 && *x_advance <= 0 )
       *x_advance = 1;
@@ -921,7 +906,7 @@
   FT_Error
   FTDemo_Draw_Index( FTDemo_Handle*   handle,
                      FTDemo_Display*  display,
-                     int              gindex,
+                     unsigned int     gindex,
                      int*             pen_x,
                      int*             pen_y )
   {
@@ -929,8 +914,13 @@
     grBitmap  bit3;
     FT_Glyph  glyf;
 
-    error = FTDemo_Index_To_Bitmap( handle, gindex, &bit3, &left, &top,
-                                    &x_advance, &y_advance, &glyf );
+
+    error = FTDemo_Index_To_Bitmap( handle,
+                                    gindex,
+                                    &bit3,
+                                    &left, &top,
+                                    &x_advance, &y_advance,
+                                    &glyf );
     if ( error )
       return error;
 
@@ -995,7 +985,6 @@
   }
 
 
-
   FT_Error
   FTDemo_Draw_Slot( FTDemo_Handle*   handle,
                     FTDemo_Display*  display,
@@ -1003,7 +992,7 @@
                     int*             pen_x,
                     int*             pen_y )
   {
-    FT_Glyph   glyph;
+    FT_Glyph  glyph;
 
 
     error = FT_Get_Glyph( slot, &glyph );
@@ -1011,8 +1000,8 @@
       return error;
 
     error = FTDemo_Draw_Glyph( handle, display, glyph, pen_x, pen_y );
-
-    FT_Done_Glyph( glyph );
+    if ( !error )
+      FT_Done_Glyph( glyph );
 
     return error;
   }
@@ -1022,12 +1011,12 @@
   FTDemo_String_Set( FTDemo_Handle*  handle,
                      const char*     string )
   {
-    const char*     p = string;
-    const char*     end = p + strlen( string );
-    unsigned long   codepoint;
-    int             ch;
-    int             expect;
-    PGlyph          glyph = handle->string;
+    const char*    p = string;
+    const char*    end = p + strlen( string );
+    unsigned long  codepoint;
+    int            ch;
+    int            expect;
+    PGlyph         glyph = handle->string;
 
 
     handle->string_length = 0;
@@ -1039,7 +1028,7 @@
       if ( ch < 0 )
         break;
 
-      codepoint = ch;
+      codepoint = (unsigned long)ch;
 
       if ( handle->encoding != FT_ENCODING_NONE )
         glyph->glyph_index = FTDemo_Get_Index( handle, codepoint );
@@ -1060,10 +1049,10 @@
   static FT_Error
   string_load( FTDemo_Handle*  handle )
   {
-    int        n;
-    FT_Size    size;
-    FT_Face    face;
-    FT_Pos     prev_rsb_delta = 0;
+    int      n;
+    FT_Size  size;
+    FT_Face  face;
+    FT_Pos   prev_rsb_delta = 0;
 
 
     error = FTDemo_Get_Size( handle, &size );
@@ -1086,7 +1075,7 @@
 
       /* load the glyph and get the image */
       if ( !FT_Load_Glyph( face, glyph->glyph_index,
-                           handle->load_flags )  &&
+                           handle->load_flags )        &&
            !FT_Get_Glyph( face->glyph, &glyph->image ) )
       {
         FT_Glyph_Metrics*  metrics = &face->glyph->metrics;
@@ -1094,7 +1083,7 @@
 
         /* note that in vertical layout, y-positive goes downwards */
 
-        glyph->vvector.x  = metrics->vertBearingX - metrics->horiBearingX;
+        glyph->vvector.x  =  metrics->vertBearingX - metrics->horiBearingX;
         glyph->vvector.y  = -metrics->vertBearingY - metrics->horiBearingY;
 
         glyph->vadvance.x = 0;
@@ -1113,7 +1102,7 @@
   }
 
 
-  FT_Error
+  static FT_Error
   string_render_prepare( FTDemo_Handle*          handle,
                          FTDemo_String_Context*  sc,
                          FT_Vector*              advances )
@@ -1124,7 +1113,7 @@
     FT_Pos      track_kern   = 0;
     FT_UInt     prev_index   = 0;
     FT_Vector*  prev_advance = NULL;
-    FT_Vector   extent       = {0, 0};
+    FT_Vector   extent       = { 0, 0 };
     FT_Int      i;
 
 
@@ -1136,17 +1125,16 @@
 
     if ( !sc->vertical && sc->kerning_degree )
     {
-      FT_Fixed  ptsize;
-
-
-      ptsize = FT_MulFix( face->units_per_EM, face->size->metrics.x_scale );
-
-      if ( FT_Get_Track_Kerning( face, ptsize << 10,
+      /* this function needs and returns points, not pixels */
+      if ( FT_Get_Track_Kerning( face,
+                                 (FT_Fixed)handle->scaler.width << 10,
                                  -sc->kerning_degree,
                                  &track_kern ) )
         track_kern = 0;
       else
-        track_kern >>= 10;
+        track_kern = (FT_Pos)(
+                       ( track_kern / 1024.0 * handle->scaler.x_res ) /
+                       72.0 );
     }
 
     for ( i = 0; i < handle->string_length; i++ )
@@ -1160,7 +1148,7 @@
         advances[i] = glyph->vadvance;
       else
       {
-        advances[i] = glyph->image->advance;
+        advances[i]     = glyph->image->advance;
         advances[i].x >>= 10;
         advances[i].y >>= 10;
 
@@ -1228,6 +1216,7 @@
     int       i, j;
     FT_Byte*  p = (FT_Byte*)bitmap->buffer;
 
+
     if ( bitmap->pitch < 0 )
       p += -bitmap->pitch * ( bitmap->rows - 1 );
 
@@ -1294,6 +1283,7 @@
     if ( sc->matrix && FT_IS_SCALABLE( face ) )
     {
       FT_Vector_Transform( &pen, sc->matrix );
+
       pen.x = ( x << 6 ) - pen.x;
       pen.y = ( y << 6 ) - pen.y;
     }
