@@ -2,13 +2,13 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2007-2013 by                                                  */
+/*  Copyright 2007-2014 by                                                  */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
 /*  FTDiff - a simple viewer to compare different hinting modes.            */
 /*                                                                          */
-/*  Press F1 when running this program to have a list of key-bindings.      */
+/*  Press ? when running this program to have a list of key-bindings.       */
 /*                                                                          */
 /****************************************************************************/
 
@@ -163,8 +163,8 @@
   static const char* const  render_mode_names[HINT_MODE_MAX] =
   {
     "unhinted",
-    "auto hinter",
-    "light auto hinter",
+    "auto-hinter",
+    "light auto-hinter",
     "native hinter"
   };
 
@@ -186,8 +186,9 @@
     unsigned char  filter_weights[5];
     int            fw_index;
 
-    int            cff_hinting_engine;
-    int            tt_interpreter_version;
+    unsigned int   cff_hinting_engine;
+    unsigned int   tt_interpreter_version;
+    FT_Bool        warping;
 
   } ColumnStateRec, *ColumnState;
 
@@ -206,13 +207,12 @@
   {
     FT_Library      library;
     const char*     text;
-    int             resolution;
-    float           char_size;
-    int             need_rescale;
+    unsigned int    resolution;
+    double          char_size;
     int             col;
     ColumnStateRec  columns[3];
     FontFace        faces;
-    int             num_faces;
+    unsigned int    num_faces;
     int             face_index;
     const char*     filepath;
     const char*     filename;
@@ -232,6 +232,7 @@
   {
     FT_UInt  cff_hinting_engine;
     FT_UInt  tt_interpreter_version;
+    FT_Bool  warping;
 
 
     memset( state, 0, sizeof ( *state ) );
@@ -253,6 +254,9 @@
     FT_Property_Get( library,
                      "truetype",
                      "interpreter-version", &tt_interpreter_version );
+    FT_Property_Get( library,
+                     "autofitter",
+                     "warping", &warping );
 
     state->columns[0].use_cboxes             = 0;
     state->columns[0].use_kerning            = 1;
@@ -262,6 +266,7 @@
     state->columns[0].hint_mode              = HINT_MODE_BYTECODE;
     state->columns[0].cff_hinting_engine     = cff_hinting_engine;
     state->columns[0].tt_interpreter_version = tt_interpreter_version;
+    state->columns[0].warping                = warping;
     state->columns[0].use_custom_lcd_filter  = 0;
     state->columns[0].fw_index               = 2;
     /* freetype default filter weights */
@@ -305,33 +310,36 @@
 
 
   static void
-  render_state_set_resolution( RenderState  state,
-                               int          resolution )
+  render_state_set_resolution( RenderState   state,
+                               unsigned int  resolution )
   {
-    state->resolution   = resolution;
-    state->need_rescale = 1;
+    state->resolution = resolution;
   }
 
 
   static void
   render_state_set_size( RenderState  state,
-                         float        char_size )
+                         double       char_size )
   {
-    state->char_size    = char_size;
-    state->need_rescale = 1;
+    state->char_size = char_size;
+  }
+
+
+  static void
+  render_state_set_face_index( RenderState  state,
+                               int          idx )
+  {
+    state->face_index = idx;
   }
 
 
   static void
   _render_state_rescale( RenderState  state )
   {
-    if ( state->need_rescale && state->size )
-    {
+    if ( state->size )
       FT_Set_Char_Size( state->face, 0,
                         (FT_F26Dot6)( state->char_size * 64.0 ),
                         0, state->resolution );
-      state->need_rescale = 0;
-    }
   }
 
 
@@ -340,9 +348,9 @@
                           char**       files,
                           char*        execname )
   {
-    FontFace  faces     = NULL;
-    int       num_faces = 0;
-    int       max_faces = 0;
+    FontFace      faces     = NULL;
+    unsigned int  num_faces = 0;
+    unsigned int  max_faces = 0;
 
 
     state->files = files;
@@ -363,7 +371,7 @@
         continue;
       }
 
-      num_subfonts = (int)face->num_faces;
+      num_subfonts = face->num_faces;
 
       FT_Done_Face( face );
 
@@ -444,31 +452,26 @@
 
 
   static int
-  render_state_set_file( RenderState  state,
-                         int          idx )
+  render_state_set_file( RenderState  state )
   {
     const char*  filepath;
 
 
-    if ( idx < 0                 ||
-         idx >= state->num_faces )
-      return -1;
-
-    state->face_index = idx;
-    filepath          = state->faces[idx].filepath;
+    filepath = state->faces[state->face_index].filepath;
 
     if ( state->face )
     {
       FT_Done_Face( state->face );
-      state->face         = NULL;
-      state->size         = NULL;
-      state->need_rescale = 1;
+      state->face = NULL;
+      state->size = NULL;
     }
 
     if ( filepath != NULL && filepath[0] != 0 )
     {
-      error = FT_New_Face( state->library, filepath,
-                           state->faces[idx].index, &state->face );
+      error = FT_New_Face( state->library,
+                           filepath,
+                           state->faces[state->face_index].index,
+                           &state->face );
       if ( error )
         return -1;
 
@@ -494,8 +497,7 @@
         state->filename = p ? p + 1 : state->filepath;
       }
 
-      state->size         = state->face->size;
-      state->need_rescale = 1;
+      state->size = state->face->size;
     }
 
     return 0;
@@ -517,7 +519,7 @@
     const char*  p              = text;
     const char*  p_end          = p + strlen( text );
     long         load_flags     = FT_LOAD_DEFAULT;
-    FT_Face      face           = state->face;
+    FT_Face      face;
     int          left           = x;
     int          right          = x + width;
     int          bottom         = y + height;
@@ -526,21 +528,29 @@
     FT_Pos       prev_rsb_delta = 0;
     FT_Pos       x_origin       = x << 6;
     HintMode     rmode          = column->hint_mode;
+    FT_Bool      warping        = column->warping;
     FT_Bool      have_0x0A      = 0;
     FT_Bool      have_0x0D      = 0;
 
 
-    if ( !face )
-      return;
-
-    _render_state_rescale( state );
-
+    /* no need to check for errors: the values used here are always valid */
     FT_Property_Set( state->library,
                      "cff",
                      "hinting-engine", &column->cff_hinting_engine );
     FT_Property_Set( state->library,
                      "truetype",
                      "interpreter-version", &column->tt_interpreter_version );
+    FT_Property_Set( state->library,
+                     "autofitter",
+                     "warping", &column->warping );
+
+    /* changing a property is in most cases a global operation; */
+    /* we are on the safe side if we reload the face completely */
+    /* (this is something a normal program doesn't need to do)  */
+    render_state_set_file( state );
+    _render_state_rescale( state );
+
+    face = state->face;
 
     if ( column->use_lcd_filter )
       FT_Library_SetLcdFilter( face->glyph->library, column->lcd_filter );
@@ -567,7 +577,7 @@
       FT_UInt       gindex;
       FT_GlyphSlot  slot = face->glyph;
       FT_Bitmap*    map  = &slot->bitmap;
-      int           xmax;
+      FT_Long       xmax;
 
 
       ch = utf8_next( &p, p_end );
@@ -624,7 +634,7 @@
         have_0x0D = 0;
       }
 
-      gindex = FT_Get_Char_Index( state->face, ch );
+      gindex = FT_Get_Char_Index( state->face, (FT_ULong)ch );
       error  = FT_Load_Glyph( face, gindex, load_flags );
 
       if ( error )
@@ -633,7 +643,7 @@
       if ( column->use_kerning && gindex != 0 && prev_glyph != 0 )
       {
         FT_Vector  vec;
-        FT_Int     kerning_mode = FT_KERNING_DEFAULT;
+        FT_UInt    kerning_mode = FT_KERNING_DEFAULT;
 
 
         if ( rmode == HINT_MODE_UNHINTED )
@@ -674,7 +684,8 @@
           xmax = ( x_origin + cbox.xMax + 63 ) >> 6;
         }
         else
-          xmax = ( x_origin >> 6 ) + slot->bitmap_left + slot->bitmap.width;
+          xmax = ( x_origin >> 6 ) +
+                 slot->bitmap_left + (FT_Long)slot->bitmap.width;
       }
       else
       {
@@ -716,7 +727,7 @@
         state->display.disp_draw( state->display.disp, mode,
                                   ( x_origin >> 6 ) + slot->bitmap_left,
                                   y - slot->bitmap_top,
-                                  map->width, map->rows,
+                                  (int)map->width, (int)map->rows,
                                   map->pitch, map->buffer );
       }
       if ( rmode == HINT_MODE_UNHINTED )
@@ -732,12 +743,12 @@
       FT_Module    module = &state->face->driver->root;
       void*        disp   = state->display.disp;
 
-      const char*  hinting_engine;
+      const char*  extra;
       const char*  msg;
       char         temp[64];
 
 
-      hinting_engine = "";
+      extra = "";
       if ( rmode == HINT_MODE_BYTECODE )
       {
         if ( !strcmp( module->clazz->module_name, "cff" ) )
@@ -745,10 +756,10 @@
           switch ( column->cff_hinting_engine )
           {
           case FT_CFF_HINTING_FREETYPE:
-            hinting_engine = " (CFF FT)";
+            extra = " (CFF FT)";
             break;
           case FT_CFF_HINTING_ADOBE:
-            hinting_engine = " (CFF Adobe)";
+            extra = " (CFF Adobe)";
             break;
           }
         }
@@ -758,17 +769,19 @@
           switch ( column->tt_interpreter_version )
           {
           case TT_INTERPRETER_VERSION_35:
-            hinting_engine = " (TT v35)";
+            extra = " (TT v35)";
             break;
           case TT_INTERPRETER_VERSION_38:
-            hinting_engine = " (TT v38)";
+            extra = " (TT v38)";
             break;
           }
         }
       }
+      else if ( rmode == HINT_MODE_AUTOHINT_LIGHT )
+        extra = warping ? " (+warp)" : " (-warp)";
 
       sprintf( temp, "%s%s",
-               render_mode_names[column->hint_mode], hinting_engine );
+               render_mode_names[column->hint_mode], extra );
       state->display.disp_text( disp, left,
                                 bottom + 5, temp );
 
@@ -925,7 +938,9 @@
       pitch = -pitch;
 
     if ( bit->mode == gr_pixel_mode_gray )
-      memset( bit->buffer, display->back_color.value, pitch * bit->rows );
+      memset( bit->buffer,
+              display->back_color.value,
+              (unsigned int)( pitch * bit->rows ) );
     else
     {
       unsigned char*  p = bit->buffer;
@@ -1019,17 +1034,32 @@
   static void
   event_help( RenderState  state )
   {
+    char  buf[256];
+    char  version[64];
+
+    const char*  format;
+    FT_Int       major, minor, patch;
+
     ADisplay  display = (ADisplay)state->display.disp;
     grEvent   dummy_event;
 
 
+    FT_Library_Version( state->library, &major, &minor, &patch );
+
+    format = patch ? "%d.%d.%d" : "%d.%d";
+    sprintf( version, format, major, minor, patch );
+
     adisplay_clear( display );
     grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( display->bitmap );
 
-    grWriteln( "FreeType hinting mode comparator - part of the FreeType test suite" );
+    sprintf( buf,
+             "FreeType Hinting Mode Comparator - part of the FreeType %s test suite",
+             version );
+
+    grWriteln( buf );
     grLn();
     grWriteln( "This program displays text using various hinting algorithms." );
     grLn();
@@ -1053,6 +1083,7 @@
     grWriteln( "  d            toggle lsb/rsb deltas" );
     grWriteln( "  h            toggle hinting mode" );
     grWriteln( "  H            cycle hinting engine (if CFF or TTF)" );
+    grWriteln( "  w            toggle warping (if light auto-hinting" );
     grWriteln( "  k            toggle kerning (only from `kern' table)" );
     grWriteln( "  r            toggle rendering mode" );
     grWriteln( "  x            toggle layout mode" );
@@ -1096,6 +1127,20 @@
   }
 
 
+  static void
+  event_change_face_index( RenderState  state,
+                           int          idx )
+  {
+    if ( idx < 0 )
+      idx = 0;
+
+    if ( idx >= (int)state->num_faces )
+      idx = (int)state->num_faces - 1;
+
+    render_state_set_face_index( state, idx );
+  }
+
+
   static int
   process_event( RenderState  state,
                  grEvent*     event )
@@ -1210,6 +1255,20 @@
       }
       break;
 
+    case grKEY( 'w' ):
+      {
+        FT_Bool  new_warping_state = !column->warping;
+
+
+        error = FT_Property_Set( state->library,
+                                 "autofitter",
+                                 "warping",
+                                 &new_warping_state );
+        if ( !error )
+          column->warping = new_warping_state;
+      }
+      break;
+
     case grKEY( 'k' ):
       column->use_kerning = !column->use_kerning;
       break;
@@ -1245,11 +1304,11 @@
       break;
 
     case grKEY( 'n' ):
-      render_state_set_file( state, state->face_index + 1 );
+      event_change_face_index( state, state->face_index + 1 );
       break;
 
     case grKEY( 'p' ):
-      render_state_set_file( state, state->face_index - 1 );
+      event_change_face_index( state, state->face_index - 1 );
       break;
 
     case grKEY( 'r' ):
@@ -1312,7 +1371,7 @@
     double    gamma    = adisplay->gamma;
     char      buf[256];
 
-    FontFace  face = &state->faces[state->face_index];
+    FontFace     face = &state->faces[state->face_index];
     const char*  basename;
 
 
@@ -1401,7 +1460,7 @@
           printf( "\n" );
           exit( 0 );
         }
-        break;
+        /* break; */
 
       case 'w':
         width = atoi( optarg );
@@ -1431,18 +1490,18 @@
         fprintf( stderr, "could not read textfile '%s'\n", textfile );
       else
       {
-        long  tsize;
+        int  tsize;
 
 
         fseek( tfile, 0, SEEK_END );
         tsize = ftell( tfile );
 
         fseek( tfile, 0, SEEK_SET );
-        text = (char*)malloc( tsize + 1 );
+        text = (char*)malloc( (unsigned int)( tsize + 1 ) );
 
         if ( text != NULL )
         {
-          fread( text, tsize, 1, tfile );
+          fread( text, (unsigned int)tsize, 1, tfile );
           text[tsize] = 0;
         }
         else
@@ -1469,15 +1528,14 @@
     render_state_init( state, display, library );
 
     if ( resolution > 0 )
-      render_state_set_resolution( state, resolution );
+      render_state_set_resolution( state, (unsigned int)resolution );
 
     if ( size > 0.0 )
       render_state_set_size( state, size );
 
     render_state_set_files( state, argv, execname );
-    render_state_set_file( state, 0 );
 
-    grSetTitle( adisplay->surface, "FreeType Text Proofer, press F1 for help" );
+    grSetTitle( adisplay->surface, "FreeType Text Proofer, press ? for help" );
 
     for (;;)
     {
@@ -1531,7 +1589,7 @@
     adisplay_done( adisplay );
     exit( 0 );  /* for safety reasons */
 
-    return 0;   /* never reached */
+    /* return 0; */  /* never reached */
   }
 
 
