@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003, 2004, 2005, 2006 by                          */
+/*  Copyright 1996-2000, 2003-2007, 2010, 2012-2013                         */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /****************************************************************************/
@@ -13,6 +13,7 @@
 #include FT_SFNT_NAMES_H
 #include FT_TRUETYPE_IDS_H
 #include FT_TRUETYPE_TABLES_H
+#include FT_MULTIPLE_MASTERS_H
 
   /* the following header shouldn't be used in normal programs */
 #include FT_INTERNAL_DEBUG_H
@@ -23,31 +24,31 @@
 #include FT_INTERNAL_DRIVER_H
 
 #include "common.h"
+#include "aux.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 
-  FT_Error  error;
+  static FT_Error  error;
 
-  int  comma_flag  = 0;
-  int  verbose     = 0;
-  int  debug       = 0;
-  int  trace_level = 0;
-  int  name_tables = 0;
+  static int  comma_flag  = 0;
+  static int  verbose     = 0;
+  static int  name_tables = 0;
+  static int  utf8        = 0;
 
 
   /* PanicZ */
-  static
-  void  PanicZ( const char*  message )
+  static void
+  PanicZ( const char*  message )
   {
     fprintf( stderr, "%s\n  error = 0x%04x\n", message, error );
     exit( 1 );
   }
 
 
-  void
+  static void
   Print_Comma( const char*  message )
   {
     if ( comma_flag )
@@ -61,27 +62,28 @@
   static void
   usage( char*  execname )
   {
-    fprintf( stderr, "\n" );
-    fprintf( stderr, "ftdump: simple font dumper -- part of the FreeType project\n" );
-    fprintf( stderr, "-----------------------------------------------------------\n" );
-    fprintf( stderr, "\n" );
-    fprintf( stderr, "Usage: %s [options] fontname\n", execname );
-    fprintf( stderr, "\n" );
-#if FREETYPE_MAJOR == 2 && FREETYPE_MINOR == 0 && FREETYPE_PATCH <= 8
-    fprintf( stderr, "  -d        enable debug information\n" );
-#  ifdef FT_DEBUG_LEVEL_TRACE
-    fprintf( stderr, "  -l level  trace level for debug information\n" );
-#  endif
-#endif
-    fprintf( stderr, "  -n        print SFNT name tables\n" );
-    fprintf( stderr, "  -v        be verbose\n" );
-    fprintf( stderr, "\n" );
+    fprintf( stderr,
+      "\n"
+      "ftdump: simple font dumper -- part of the FreeType project\n"
+      "-----------------------------------------------------------\n"
+      "\n"
+      "Usage: %s [options] fontname\n"
+      "\n",
+             execname );
+
+    fprintf( stderr,
+      "  -n        Print SFNT name tables.\n"
+      "  -u        Emit UTF8.\n"
+      "  -V        Be verbose.\n"
+      "\n"
+      "  -v        Show version.\n"
+      "\n" );
 
     exit( 1 );
   }
 
 
-  void
+  static void
   Print_Name( FT_Face  face )
   {
     const char*  ps_name;
@@ -102,7 +104,7 @@
   }
 
 
-  void
+  static void
   Print_Type( FT_Face  face )
   {
     FT_Module  module;
@@ -205,7 +207,7 @@
     case TT_NAME_ID_TRADEMARK:
       return "trademark";
 
-   /* the following values are from the OpenType spec */
+    /* the following values are from the OpenType spec */
     case TT_NAME_ID_MANUFACTURER:
       return "manufacturer";
     case TT_NAME_ID_DESIGNER:
@@ -228,143 +230,27 @@
     case TT_NAME_ID_MAC_FULL_NAME:
       return "Mac full name";
 
-   /* The following code is new as of 2000-01-21 */
+    /* the following code is new as of 2000-01-21 */
     case TT_NAME_ID_SAMPLE_TEXT:
       return "sample text";
 
-   /* This is new in OpenType 1.3 */
+    /* this is new in OpenType 1.3 */
     case TT_NAME_ID_CID_FINDFONT_NAME:
       return "CID `findfont' name";
 
+    /* this is new in OpenType 1.5 */
+    case TT_NAME_ID_WWS_FAMILY:
+      return "WWS family name";
+    case TT_NAME_ID_WWS_SUBFAMILY:
+      return "WWS subfamily name";
+
     default:
-      return "UNKNOWN";
+      return NULL;
     }
   }
 
 
   static void
-  put_ascii( FT_Byte*  string,
-             FT_UInt   string_len,
-             FT_UInt   indent )
-  {
-    FT_UInt  i, j;
-
-
-    for ( j = 0; j < indent; j++ )
-      putchar( ' ' );
-    putchar( '"' );
-
-    for ( i = 0; i < string_len; i++ )
-    {
-      switch ( string[i] )
-      {
-      case '\n':
-        fputs( "\\n\"", stdout );
-        if ( i + 1 < string_len )
-        {
-          putchar( '\n' );
-          for ( j = 0; j < indent; j++ )
-            putchar( ' ' );
-          putchar( '"' );
-        }
-        break;
-      case '\r':
-        fputs( "\\r", stdout );
-        break;
-      case '\t':
-        fputs( "\\t", stdout );
-        break;
-      case '\\':
-        fputs( "\\\\", stdout );
-        break;
-      case '"':
-        fputs( "\\\"", stdout );
-        break;
-
-      default:
-        putchar( string[i] );
-        break;
-      }
-    }
-    if ( string[i - 1] != '\n' )
-      putchar( '"' );
-  }
-
-
-  static void
-  put_unicode_be16( FT_Byte*  string,
-                    FT_UInt   string_len,
-                    FT_UInt   indent )
-  {
-    FT_Int   ch = 0;
-    FT_UInt  i, j;
-
-
-    for ( j = 0; j < indent; j++ )
-      putchar( ' ' );
-    putchar( '"' );
-
-    for ( i = 0; i < string_len; i += 2 )
-    {
-      ch = ( string[i] << 8 ) | string[i + 1];
-
-      switch ( ch )
-      {
-      case '\n':
-        fputs( "\\n\"", stdout );
-        if ( i + 2 < string_len )
-        {
-          putchar( '\n' );
-          for ( j = 0; j < indent; j++ )
-            putchar( ' ' );
-          putchar( '"' );
-        }
-        break;
-      case '\r':
-        fputs( "\\r", stdout );
-        break;
-      case '\t':
-        fputs( "\\t", stdout );
-        break;
-      case '\\':
-        fputs( "\\\\", stdout );
-        break;
-      case '"':
-        fputs( "\\\"", stdout );
-        break;
-
-      case 0x00A9:
-        fputs( "(c)", stdout );
-        break;
-      case 0x00AE:
-        fputs( "(r)", stdout );
-        break;
-
-      case 0x2013:
-        fputs( "--", stdout );
-        break;
-      case 0x2019:
-        fputs( "\'", stdout );
-        break;
-
-      case 0x2122:
-        fputs( "(tm)", stdout );
-        break;
-
-      default:
-        if ( ch < 256 )
-          putchar( ch );
-        else
-          printf( "\\U+%04X", ch );
-        break;
-      }
-    }
-    if ( ch != '\n' )
-      putchar( '"' );
-  }
-
-
-  void
   Print_Sfnt_Names( FT_Face  face )
   {
     FT_SfntName  name;
@@ -379,30 +265,37 @@
       error = FT_Get_Sfnt_Name( face, i, &name );
       if ( error == FT_Err_Ok )
       {
-        printf( "   %-15s [%s]", name_id( name.name_id ),
-                                 platform_id( name.platform_id ) );
+        const char*  NameID     = name_id( name.name_id );
+        const char*  PlatformID = platform_id( name.platform_id );
+
+
+        if ( NameID )
+          printf( "   %-15s [%s]", NameID, PlatformID );
+        else
+          printf( "   Name ID %-5d   [%s]", name.name_id, PlatformID );
 
         switch ( name.platform_id )
         {
         case TT_PLATFORM_APPLE_UNICODE:
+          fputs( ":\n", stdout );
           switch ( name.encoding_id )
           {
           case TT_APPLE_ID_DEFAULT:
           case TT_APPLE_ID_UNICODE_1_1:
           case TT_APPLE_ID_ISO_10646:
           case TT_APPLE_ID_UNICODE_2_0:
-            put_unicode_be16( name.string, name.string_len, 6 );
+            put_unicode_be16( name.string, name.string_len, 6, utf8 );
             break;
 
           default:
-            printf( "{unsupported encoding %d}", name.encoding_id );
+            printf( "{unsupported Unicode encoding %d}", name.encoding_id );
             break;
           }
           break;
 
         case TT_PLATFORM_MACINTOSH:
           if ( name.language_id != TT_MAC_LANGID_ENGLISH )
-            printf( " (language=%d)", name.language_id );
+            printf( " (language=%u)", name.language_id );
           fputs( ":\n", stdout );
 
           switch ( name.encoding_id )
@@ -415,13 +308,14 @@
             break;
 
           default:
-            printf( "{unsupported encoding %d}", name.encoding_id );
+            printf( "      [data in encoding %d]", name.encoding_id );
             break;
           }
 
           break;
 
         case TT_PLATFORM_ISO:
+          fputs( ":\n", stdout );
           switch ( name.encoding_id )
           {
           case TT_ISO_ID_7BIT_ASCII:
@@ -430,7 +324,7 @@
             break;
 
           case TT_ISO_ID_10646:
-            put_unicode_be16( name.string, name.string_len, 6 );
+            put_unicode_be16( name.string, name.string_len, 6, utf8 );
             break;
 
           default:
@@ -446,11 +340,11 @@
 
           switch ( name.encoding_id )
           {
-          /* TT_MS_ID_SYMBOL_CS is supposed to be Unicode, according to */
-          /* information from the MS font development team              */
+            /* TT_MS_ID_SYMBOL_CS is supposed to be Unicode, according to */
+            /* information from the MS font development team              */
           case TT_MS_ID_SYMBOL_CS:
           case TT_MS_ID_UNICODE_CS:
-            put_unicode_be16( name.string, name.string_len, 6 );
+            put_unicode_be16( name.string, name.string_len, 6, utf8 );
             break;
 
           default:
@@ -471,7 +365,7 @@
   }
 
 
-  void
+  static void
   Print_Fixed( FT_Face  face )
   {
     int  i;
@@ -495,7 +389,7 @@
   }
 
 
-  void
+  static void
   Print_Charmaps( FT_Face  face )
   {
     int  i, active = -1;
@@ -509,13 +403,23 @@
 
     for( i = 0; i < face->num_charmaps; i++ )
     {
-      printf( "   %d: platform %d, encoding %d, language %d",
+      FT_ULong  lang_id = FT_Get_CMap_Language_ID( face->charmaps[i] );
+
+
+      printf( "   %d: platform %u, encoding %2u",
               i,
               face->charmaps[i]->platform_id,
-              face->charmaps[i]->encoding_id,
-              (FT_UInt)FT_Get_CMap_Language_ID( face->charmaps[i] ) );
+              face->charmaps[i]->encoding_id );
+
+      if ( lang_id == 0xFFFFFFFFUL )
+        printf( "   (Unicode Variation Sequences)" );
+      else
+        printf( "   language %lu",
+                lang_id );
+
       if ( i == active )
         printf( " (active)" );
+
       printf ( "\n" );
 
       if ( verbose )
@@ -538,13 +442,92 @@
   }
 
 
+  static void
+  Print_MM_Axes( FT_Face  face )
+  {
+    FT_MM_Var*       mm;
+    FT_Multi_Master  dummy;
+    FT_UInt          is_GX, i, num_names;
+
+
+    /* MM or GX axes */
+    error = FT_Get_Multi_Master( face, &dummy );
+    is_GX = error ? 1 : 0;
+
+    printf( "%s axes\n", is_GX ? "GX" : "MM" );
+
+    error = FT_Get_MM_Var( face, &mm );
+    if ( error )
+    {
+      printf( "   Can't access axis data (error code %d)\n", error );
+      return;
+    }
+
+    num_names = FT_Get_Sfnt_Name_Count( face );
+
+    for ( i = 0; i < mm->num_axis; i++ )
+    {
+      FT_SfntName  name;
+
+
+      name.string = NULL;
+
+      if ( is_GX )
+      {
+        FT_UInt  strid = mm->axis[i].strid;
+        FT_UInt  j;
+
+
+        /* iterate over all name entries        */
+        /* to find an English entry for `strid' */
+
+        for ( j = 0; j < num_names; j++ )
+        {
+          error = FT_Get_Sfnt_Name( face, j, &name );
+          if ( error )
+            continue;
+
+          if ( name.name_id == strid )
+          {
+            /* XXX we don't have support for Apple's new `ltag' table yet, */
+            /* thus we ignore TT_PLATFORM_APPLE_UNICODE                    */
+            if ( ( name.platform_id == TT_PLATFORM_MACINTOSH &&
+                   name.language_id == TT_MAC_LANGID_ENGLISH )        ||
+                 ( name.platform_id == TT_PLATFORM_MICROSOFT        &&
+                   ( name.language_id & 0xFF )
+                                    == TT_MS_LANGID_ENGLISH_GENERAL ) )
+              break;
+          }
+        }
+      }
+
+      if ( name.string )
+      {
+        if ( name.platform_id == TT_PLATFORM_MACINTOSH )
+          put_ascii( name.string, name.string_len, 3 );
+        else
+          put_unicode_be16( name.string, name.string_len, 3, utf8 );
+      }
+      else
+        printf( "   %s", mm->axis[i].name );
+
+      printf( ": [%g;%g], default %g\n",
+              mm->axis[i].minimum / 65536.0,
+              mm->axis[i].maximum / 65536.0,
+              mm->axis[i].def / 65536.0 );
+    }
+
+    free( mm );
+  }
+
+
   int
   main( int    argc,
         char*  argv[] )
   {
     int    i, file;
-    char   filename[128 + 4];
-    char   alt_filename[128 + 4];
+    char   filename[1024 + 4];
+    char   alt_filename[1024 + 4];
     char*  execname;
     int    num_faces;
     int    option;
@@ -555,30 +538,44 @@
 
     execname = ft_basename( argv[0] );
 
+    /* Initialize engine */
+    error = FT_Init_FreeType( &library );
+    if ( error )
+      PanicZ( "Could not initialize FreeType library" );
+
     while ( 1 )
     {
-      option = getopt( argc, argv, "dl:nv" );
+      option = getopt( argc, argv, "nuvV" );
 
       if ( option == -1 )
         break;
 
       switch ( option )
       {
-      case 'd':
-        debug = 1;
-        break;
-
-      case 'l':
-        trace_level = atoi( optarg );
-        if ( trace_level < 1 || trace_level > 7 )
-          usage( execname );
-        break;
-
       case 'n':
         name_tables = 1;
         break;
 
+      case 'u':
+        utf8 = 1;
+        break;
+
       case 'v':
+        {
+          FT_Int  major, minor, patch;
+
+
+          FT_Library_Version( library, &major, &minor, &patch );
+
+          printf( "ftdump (FreeType) %d.%d", major, minor );
+          if ( patch )
+            printf( ".%d", patch );
+          printf( "\n" );
+          exit( 0 );
+        }
+        /* break; */
+
+      case 'V':
         verbose = 1;
         break;
 
@@ -594,40 +591,13 @@
     if ( argc != 1 )
       usage( execname );
 
-#if FREETYPE_MAJOR == 2 && FREETYPE_MINOR == 0 && FREETYPE_PATCH <= 8
-    if ( debug )
-    {
-#  ifdef FT_DEBUG_LEVEL_TRACE
-      FT_SetTraceLevel( trace_any, (FT_Byte)trace_level );
-#  else
-      trace_level = 0;
-#  endif
-    }
-#elif 0
-       /* "setenv/putenv" is not ANSI and I don't want to mess */
-       /* with this portability issue right now                */
-    if ( debug )
-    {
-      char  temp[32];
-
-
-      sprintf( temp, "any=%d", trace_level );
-      setenv( "FT2_DEBUG", temp );
-    }
-#endif
-
     file = 0;
 
-    /* Initialize engine */
-    error = FT_Init_FreeType( &library );
-    if ( error )
-      PanicZ( "Could not initialize FreeType library" );
+    filename[1024]     = '\0';
+    alt_filename[1024] = '\0';
 
-    filename[128]     = '\0';
-    alt_filename[128] = '\0';
-
-    strncpy( filename, argv[file], 128 );
-    strncpy( alt_filename, argv[file], 128 );
+    strncpy( filename, argv[file], 1024 );
+    strncpy( alt_filename, argv[file], 1024 );
 
     /* try to load the file name as is, first */
     error = FT_New_Face( library, argv[file], 0, &face );
@@ -635,7 +605,7 @@
       goto Success;
 
 #ifndef macintosh
-    i = strlen( argv[file] );
+    i = (int)strlen( argv[file] );
     while ( i > 0 && argv[file][i] != '\\' && argv[file][i] != '/' )
     {
       if ( argv[file][i] == '.' )
@@ -675,7 +645,7 @@
       printf( "\n" );
       Print_Type( face );
 
-      printf( "   glyph count: %ld\n", face->num_glyphs );
+      printf( "   glyph count:     %ld\n", face->num_glyphs );
 
       if ( name_tables && FT_IS_SFNT( face ) )
       {
@@ -695,13 +665,19 @@
         Print_Charmaps( face );
       }
 
+      if ( FT_HAS_MULTIPLE_MASTERS( face ) )
+      {
+        printf( "\n" );
+        Print_MM_Axes( face );
+      }
+
       FT_Done_Face( face );
     }
 
     FT_Done_FreeType( library );
 
     exit( 0 );      /* for safety reasons */
-    return 0;       /* never reached */
+    /* return 0; */ /* never reached */
   }
 
 
