@@ -2,13 +2,13 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003-2005, 2010-2013 by                            */
+/*  Copyright 1996-2000, 2003-2005, 2010-2014 by                            */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
 /*  FTMulti- a simple multiple masters font viewer                          */
 /*                                                                          */
-/*  Press F1 when running this program to have a list of key-bindings       */
+/*  Press ? when running this program to have a list of key-bindings        */
 /*                                                                          */
 /****************************************************************************/
 
@@ -29,62 +29,58 @@
 #define  DIM_X   640
 #define  DIM_Y   480
 
-#define  CENTER_X   ( bit.width / 2 )
-#define  CENTER_Y   ( bit.rows / 2 )
+#define  MAXPTSIZE    500               /* dtp */
+#define  MAX_MM_AXES    6
 
-#define  MAXPTSIZE  500                 /* dtp */
 
-  char  Header[256];
-  char* new_header = 0;
+  static char  Header[256];
+  static char* new_header = 0;
 
-  const unsigned char*  Text = (unsigned char*)
+  static const unsigned char*  Text = (unsigned char*)
     "The quick brown fox jumps over the lazy dog 0123456789 "
     "\342\352\356\373\364\344\353\357\366\374\377\340\371\351\350\347 "
     "&#~\"\'(-`_^@)=+\260 ABCDEFGHIJKLMNOPQRSTUVWXYZ "
     "$\243^\250*\265\371%!\247:/;.,?<>";
 
-  FT_Library    library;      /* the FreeType library        */
-  FT_Face       face;         /* the font face               */
-  FT_Size       size;         /* the font size               */
-  FT_GlyphSlot  glyph;        /* the glyph slot              */
+  static FT_Library    library;      /* the FreeType library        */
+  static FT_Face       face;         /* the font face               */
+  static FT_Size       size;         /* the font size               */
+  static FT_GlyphSlot  glyph;        /* the glyph slot              */
 
-  FT_Encoding   encoding = FT_ENCODING_NONE;
+  static FT_Encoding   encoding = FT_ENCODING_NONE;
 
-  FT_Error      error;        /* error returned by FreeType? */
+  static FT_Error      error;        /* error returned by FreeType? */
 
-  grSurface*    surface;      /* current display surface     */
-  grBitmap      bit;          /* current display bitmap      */
+  static grSurface*    surface;      /* current display surface     */
+  static grBitmap      bit;          /* current display bitmap      */
 
-  int  width     = DIM_X;     /* window width                */
-  int  height    = DIM_Y;     /* window height               */
+  static int  width     = DIM_X;     /* window width                */
+  static int  height    = DIM_Y;     /* window height               */
 
-  int  num_glyphs;            /* number of glyphs            */
-  int  ptsize;                /* current point size          */
+  static int  num_glyphs;            /* number of glyphs            */
+  static int  ptsize;                /* current point size          */
 
-  int  hinted    = 1;         /* is glyph hinting active?    */
-  int  antialias = 1;         /* is anti-aliasing active?    */
-  int  use_sbits = 1;         /* do we use embedded bitmaps? */
-  int  Num;                   /* current first glyph index   */
+  static int  hinted    = 1;         /* is glyph hinting active?    */
+  static int  antialias = 1;         /* is anti-aliasing active?    */
+  static int  use_sbits = 1;         /* do we use embedded bitmaps? */
+  static int  Num;                   /* current first glyph index   */
 
-  int  res       = 72;
+  static int  res       = 72;
 
   static grColor  fore_color = { 255 };
 
-  int            Fail;
-  unsigned char  autorun;
+  static int  Fail;
 
-  int  graph_init  = 0;
+  static int  graph_init  = 0;
 
-  int  render_mode = 1;
-  int  use_grays   = 1;
+  static int  render_mode = 1;
 
-  FT_MM_Var       *multimaster = NULL;
-  FT_Fixed         design_pos   [T1_MAX_MM_AXIS];
-  FT_Fixed         requested_pos[T1_MAX_MM_AXIS];
-  int              requested_cnt = 0;
+  static FT_MM_Var  *multimaster   = NULL;
+  static FT_Fixed    design_pos   [MAX_MM_AXES];
+  static FT_Fixed    requested_pos[MAX_MM_AXES];
+  static int         requested_cnt = 0;
+  static int         used_num_axis = 0;
 
-#define RASTER_BUFF_SIZE  32768
-  char             raster_buff[RASTER_BUFF_SIZE];
 
 #define DEBUGxxx
 
@@ -94,6 +90,7 @@
 #define LOG( x )  /* empty */
 #endif
 
+
 #ifdef DEBUG
   static void
   LogMessage( const char*  fmt, ... )
@@ -139,7 +136,7 @@
   static void
   parse_design_coords( char  *s )
   {
-    for ( requested_cnt = 0; requested_cnt < T1_MAX_MM_AXIS && *s;
+    for ( requested_cnt = 0; requested_cnt < MAX_MM_AXES && *s;
           requested_cnt++ )
     {
       requested_pos[requested_cnt] = (FT_Fixed)( strtod( s, &s ) * 65536.0 );
@@ -159,7 +156,7 @@
 
     if ( bitmap_size < 0 )
       bitmap_size = -bitmap_size;
-    memset( bit.buffer, 0, bitmap_size );
+    memset( bit.buffer, 0, (unsigned long)bitmap_size );
   }
 
 
@@ -182,11 +179,6 @@
   }
 
 
-#define FLOOR( x )  ( (x) & -64 )
-#define CEIL( x )   ( ( (x) + 63 ) & -64 )
-#define TRUNC( x )  ( (x) >> 6 )
-
-
   /* Render a single glyph with the `grays' component */
   static FT_Error
   Render_Glyph( int  x_offset,
@@ -206,8 +198,8 @@
     }
 
     /* now blit it to our display screen */
-    bit3.rows   = glyph->bitmap.rows;
-    bit3.width  = glyph->bitmap.width;
+    bit3.rows   = (int)glyph->bitmap.rows;
+    bit3.width  = (int)glyph->bitmap.width;
     bit3.pitch  = glyph->bitmap.pitch;
     bit3.buffer = glyph->bitmap.buffer;
 
@@ -236,13 +228,15 @@
   static void
   Reset_Scale( int  pointSize )
   {
-    (void)FT_Set_Char_Size( face, pointSize << 6, pointSize << 6, res, res );
+    (void)FT_Set_Char_Size( face,
+                            pointSize << 6, pointSize << 6,
+                            (FT_UInt)res, (FT_UInt)res );
   }
 
 
   static FT_Error
-  LoadChar( int  idx,
-            int  hint )
+  LoadChar( unsigned int  idx,
+            int           hint )
   {
     int  flags;
 
@@ -260,15 +254,15 @@
 
 
   static FT_Error
-  Render_All( int  first_glyph,
-              int  pt_size )
+  Render_All( unsigned int  first_glyph,
+              int           pt_size )
   {
-    FT_F26Dot6  start_x, start_y, step_y, x, y;
-    int         i;
+    FT_F26Dot6    start_x, start_y, step_y, x, y;
+    unsigned int  i;
 
 
     start_x = 4;
-    start_y = 36 + pt_size;
+    start_y = pt_size + ( used_num_axis > MAX_MM_AXES / 2 ? 44 : 36 );
 
     step_y = size->metrics.y_ppem + 10;
 
@@ -278,9 +272,9 @@
     i = first_glyph;
 
 #if 0
-     while ( i < first_glyph + 1 )
+    while ( i < first_glyph + 1 )
 #else
-     while ( i < num_glyphs )
+    while ( i < (unsigned int)num_glyphs )
 #endif
     {
       if ( !( error = LoadChar( i, hinted ) ) )
@@ -322,10 +316,10 @@
 
 
   static FT_Error
-  Render_Text( int  first_glyph )
+  Render_Text( unsigned int  first_glyph )
   {
-    FT_F26Dot6  start_x, start_y, step_y, x, y;
-    int         i;
+    FT_F26Dot6    start_x, start_y, step_y, x, y;
+    unsigned int  i;
 
     const unsigned char*  p;
 
@@ -392,49 +386,60 @@
   static void
   Help( void )
   {
+    char  buf[256];
+    char  version[64];
+
+    const char*  format;
+    FT_Int       major, minor, patch;
+
     grEvent  dummy_event;
 
 
+    FT_Library_Version( library, &major, &minor, &patch );
+
+    format = patch ? "%d.%d.%d" : "%d.%d";
+    sprintf( version, format, major, minor, patch );
+
     Clear_Display();
     grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( &bit );
 
-    grWriteln( "FreeType Multiple Masters Glyph Viewer - part of the FreeType test suite" );
+    sprintf( buf,
+             "FreeType MM Glyph Viewer - part of the FreeType %s test suite",
+             version );
+
+    grWriteln( buf );
     grLn();
-    grWriteln( "This program is used to display all glyphs from one or" );
-    grWriteln( "several Multiple Masters font files, with the FreeType library.");
+    grWriteln( "This program displays all glyphs from one or several" );
+    grWriteln( "Multiple Masters or GX font files, with the FreeType library." );
     grLn();
     grWriteln( "Use the following keys:");
     grLn();
-    grWriteln( "  F1 or ?   : display this help screen" );
-    grWriteln( "  a         : toggle anti-aliasing" );
-    grWriteln( "  h         : toggle outline hinting" );
-    grWriteln( "  b         : toggle embedded bitmaps" );
-    grWriteln( "  space     : toggle rendering mode" );
+    grWriteln( "?           display this help screen" );
+    grWriteln( "a           toggle anti-aliasing" );
+    grWriteln( "h           toggle outline hinting" );
+    grWriteln( "b           toggle embedded bitmaps" );
+    grWriteln( "space       toggle rendering mode" );
     grLn();
-    grWriteln( "  n         : next font" );
-    grWriteln( "  p         : previous font" );
+    grWriteln( "p, n        previous/next font" );
     grLn();
-    grWriteln( "  Up        : increase pointsize by 1 unit" );
-    grWriteln( "  Down      : decrease pointsize by 1 unit" );
-    grWriteln( "  Page Up   : increase pointsize by 10 units" );
-    grWriteln( "  Page Down : decrease pointsize by 10 units" );
+    grWriteln( "Up, Down    change pointsize by 1 unit" );
+    grWriteln( "PgUp, PgDn  change pointsize by 10 units" );
     grLn();
-    grWriteln( "  Right     : increment first glyph index" );
-    grWriteln( "  Left      : decrement first glyph index" );
+    grWriteln( "Left, Right adjust index by 1" );
+    grWriteln( "F7, F8      adjust index by 10" );
+    grWriteln( "F9, F10     adjust index by 100" );
+    grWriteln( "F11, F12    adjust index by 1000" );
+    grLn();
+    grWriteln( "F1, F2      adjust first axis by 1/50th of its range" );
+    grWriteln( "F3, F4      adjust second axis by 1/50th of its range" );
+    grWriteln( "F5, F6      adjust third axis by 1/50th of its range" );
+    grWriteln( "1, 2        adjust fourth axis by 1/50th of its range" );
+    grWriteln( "3, 4        adjust fifth axis by 1/50th of its range" );
+    grWriteln( "5, 6        adjust sixth axis by 1/50th of its range" );
     grLn();
-    grWriteln( "  F3        : decrement first axis pos by 1/50th of its range" );
-    grWriteln( "  F4        : increment first axis pos by 1/50th of its range" );
-    grWriteln( "  F5        : decrement second axis pos by 1/50th of its range" );
-    grWriteln( "  F6        : increment second axis pos by 1/50th of its range" );
-    grWriteln( "  F7        : decrement third axis pos by 1/50th of its range" );
-    grWriteln( "  F8        : increment third axis pos by 1/50th of its range" );
-    grWriteln( "  F9        : decrement index by 100" );
-    grWriteln( "  F10       : increment index by 100" );
-    grWriteln( "  F11       : decrement index by 1000" );
-    grWriteln( "  F12       : increment index by 1000" );
     grLn();
     grWriteln( "press any key to exit this help screen" );
 
@@ -455,7 +460,6 @@
     case grKEY( 'q' ):
       return 0;
 
-    case grKeyF1:
     case grKEY( '?' ):
       Help();
       return 1;
@@ -487,40 +491,70 @@
 
     case grKEY( ' ' ):
       render_mode ^= 1;
-      new_header = render_mode ? (char *)"rendering all glyphs in font"
-                               : (char *)"rendering test text string";
+      new_header   = render_mode ? (char *)"rendering all glyphs in font"
+                                 : (char *)"rendering test text string";
       break;
 
     /* MM related keys */
 
-    case grKeyF3:
+    case grKeyF1:
       i = -20;
       axis = 0;
       goto Do_Axis;
 
+    case grKeyF2:
+      i = 20;
+      axis = 0;
+      goto Do_Axis;
+
+    case grKeyF3:
+      i = -20;
+      axis = 1;
+      goto Do_Axis;
+
     case grKeyF4:
       i = 20;
-      axis = 0;
+      axis = 1;
       goto Do_Axis;
 
     case grKeyF5:
       i = -20;
-      axis = 1;
+      axis = 2;
       goto Do_Axis;
 
     case grKeyF6:
       i = 20;
-      axis = 1;
+      axis = 2;
       goto Do_Axis;
 
-    case grKeyF7:
+    case grKEY( '1' ):
       i = -20;
-      axis = 2;
+      axis = 3;
       goto Do_Axis;
 
-    case grKeyF8:
+    case grKEY( '2' ):
       i = 20;
-      axis = 2;
+      axis = 3;
+      goto Do_Axis;
+
+    case grKEY( '3' ):
+      i = -20;
+      axis = 4;
+      goto Do_Axis;
+
+    case grKEY( '4' ):
+      i = 20;
+      axis = 4;
+      goto Do_Axis;
+
+    case grKEY( '5' ):
+      i = -20;
+      axis = 5;
+      goto Do_Axis;
+
+    case grKEY( '6' ):
+      i = 20;
+      axis = 5;
       goto Do_Axis;
 
     /* scaling related keys */
@@ -551,6 +585,14 @@
       i = 1;
       goto Do_Glyph;
 
+    case grKeyF7:
+      i = -10;
+      goto Do_Glyph;
+
+    case grKeyF8:
+      i = 10;
+      goto Do_Glyph;
+
     case grKeyF9:
       i = -100;
       goto Do_Glyph;
@@ -573,22 +615,25 @@
     return 1;
 
   Do_Axis:
-    if ( axis < (int)multimaster->num_axis )
+    if ( axis < used_num_axis )
     {
-      FT_Var_Axis* a   = multimaster->axis + axis;
-      FT_Fixed     pos = design_pos[axis];
+      FT_Var_Axis*  a   = multimaster->axis + axis;
+      FT_Fixed      pos = design_pos[axis];
 
-      /* Normalize i. changing by 20 is all very well for PostScript fonts  */
-      /*  which tend to have a range of ~1000 per axis, but it's not useful */
-      /*  for mac fonts which have a range of ~3.                           */
-      /* And it's rather extreme for optical size even in PS                */
-      pos += FT_MulDiv( i, a->maximum-a->minimum, 1000 );
+
+      /*
+       * Normalize i.  Changing by 20 is all very well for PostScript fonts,
+       * which tend to have a range of ~1000 per axis, but it's not useful
+       * for mac fonts, which have a range of ~3.  And it's rather extreme
+       * for optical size even in PS.
+       */
+      pos += FT_MulDiv( i, a->maximum - a->minimum, 1000 );
       if ( pos < a->minimum ) pos = a->minimum;
       if ( pos > a->maximum ) pos = a->maximum;
 
       design_pos[axis] = pos;
 
-      FT_Set_Var_Design_Coordinates( face, multimaster->num_axis, design_pos );
+      FT_Set_Var_Design_Coordinates( face, used_num_axis, design_pos );
     }
     return 1;
 
@@ -714,7 +759,7 @@
           printf( "\n" );
           exit( 0 );
         }
-        break;
+        /* break; */
 
       case 'w':
         width = atoi( optarg );
@@ -747,7 +792,10 @@
     /* Load face */
     error = FT_New_Face( library, argv[file], 0, &face );
     if ( error )
+    {
+      face = NULL;
       goto Display_Font;
+    }
 
     if ( encoding != FT_ENCODING_NONE )
     {
@@ -759,7 +807,10 @@
     /* retrieve multiple master information */
     error = FT_Get_MM_Var( face, &multimaster );
     if ( error )
+    {
+      multimaster = NULL;
       goto Display_Font;
+    }
 
     /* if the user specified a position, use it, otherwise */
     /* set the current position to the median of each axis */
@@ -767,7 +818,16 @@
       int  n;
 
 
-      for ( n = 0; n < (int)multimaster->num_axis; n++ )
+      if ( multimaster->num_axis > MAX_MM_AXES )
+      {
+        fprintf( stderr, "only handling first %d GX axes (of %d)\n",
+                         MAX_MM_AXES, multimaster->num_axis );
+        used_num_axis = MAX_MM_AXES;
+      }
+      else
+        used_num_axis = (int)multimaster->num_axis;
+
+      for ( n = 0; n < used_num_axis; n++ )
       {
         design_pos[n] = n < requested_cnt ? requested_pos[n]
                                           : multimaster->axis[n].def;
@@ -778,9 +838,7 @@
       }
     }
 
-    error = FT_Set_Var_Design_Coordinates( face,
-                                           multimaster->num_axis,
-                                           design_pos );
+    error = FT_Set_Var_Design_Coordinates( face, used_num_axis, design_pos );
     if ( error )
       goto Display_Font;
 
@@ -800,7 +858,7 @@
       Init_Display();
     }
 
-    grSetTitle( surface, "FreeType Glyph Viewer - press F1 for help" );
+    grSetTitle( surface, "FreeType Glyph Viewer - press ? for help" );
     old_ptsize = ptsize;
 
     if ( file_loaded >= 1 )
@@ -827,11 +885,11 @@
         switch ( render_mode )
         {
         case 0:
-          Render_Text( Num );
+          Render_Text( (unsigned int)Num );
           break;
 
         default:
-          Render_All( Num, ptsize );
+          Render_All( (unsigned int)Num, ptsize );
         }
 
         sprintf( Header, "%.50s %.50s (file %.100s)",
@@ -845,17 +903,19 @@
         grWriteCellString( &bit, 0, 0, new_header, fore_color );
         new_header = 0;
 
-        sprintf( Header, "axis: " );
+        sprintf( Header, "axes:" );
         {
           int  n;
+          int  limit = used_num_axis > MAX_MM_AXES / 2 ? MAX_MM_AXES / 2
+                                                       : used_num_axis;
 
 
-          for ( n = 0; n < (int)multimaster->num_axis; n++ )
+          for ( n = 0; n < limit; n++ )
           {
             char  temp[100];
 
 
-            sprintf( temp, "  %.50s:%g",
+            sprintf( temp, "  %.50s: %.02f",
                            multimaster->axis[n].name,
                            design_pos[n] / 65536.0 );
             strncat( Header, temp,
@@ -864,6 +924,29 @@
         }
         grWriteCellString( &bit, 0, 16, Header, fore_color );
 
+        if ( used_num_axis > MAX_MM_AXES / 2 )
+        {
+          int  n;
+          int  limit = used_num_axis;
+
+
+          sprintf( Header, "     " );
+
+          for ( n = MAX_MM_AXES / 2; n < limit; n++ )
+          {
+            char  temp[100];
+
+
+            sprintf( temp, "  %.50s: %.02f",
+                           multimaster->axis[n].name,
+                           design_pos[n] / 65536.0 );
+            strncat( Header, temp,
+                     sizeof ( Header ) - strlen( Header ) - 1 );
+          }
+
+          grWriteCellString( &bit, 0, 24, Header, fore_color );
+        }
+
         sprintf( Header, "at %d points, first glyph = %d",
                          ptsize,
                          Num );
@@ -923,7 +1006,7 @@
     printf( "Fails = %d\n", Fail );
 
     exit( 0 );      /* for safety reasons */
-    return 0;       /* never reached */
+    /* return 0; */ /* never reached */
   }
 
 
