@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2007, 2009-2013 by                                       */
+/*  Copyright 1996-2007, 2009-2014 by                                       */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -73,16 +73,31 @@
   static void
   event_help( void )
   {
+    char  buf[256];
+    char  version[64];
+
+    const char*  format;
+    FT_Int       major, minor, patch;
+
     grEvent  dummy_event;
 
 
+    FT_Library_Version( handle->library, &major, &minor, &patch );
+
+    format = patch ? "%d.%d.%d" : "%d.%d";
+    sprintf( version, format, major, minor, patch );
+
     FTDemo_Display_Clear( display );
     grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( display->bitmap );
 
-    grWriteln( "FreeType String Viewer - part of the FreeType test suite" );
+    sprintf( buf,
+             "FreeType String Viewer - part of the FreeType %s test suite",
+             version );
+
+    grWriteln( buf );
     grLn();
     grWriteln( "This program is used to display a string of text using" );
     grWriteln( "the new convenience API of the FreeType 2 library." );
@@ -540,7 +555,7 @@
           printf( "\n" );
           exit( 0 );
         }
-        break;
+        /* break; */
 
       case 'w':
         status.width = atoi( optarg );
@@ -611,7 +626,7 @@
       PanicZ( "could not allocate display surface" );
 
     grSetTitle( display->surface,
-                "FreeType String Viewer - press F1 for help" );
+                "FreeType String Viewer - press ? for help" );
 
     event_gamma_change( 0 );
     event_font_change( 0 );
@@ -635,7 +650,7 @@
         {
           FTDemo_String_Context  sc = status.sc;
           FT_Int                 x, y;
-          FT_UInt                height;
+          FT_Int                 height;
 
 
           x = 55;
@@ -695,7 +710,7 @@
     FTDemo_Done( handle );
     exit( 0 );      /* for safety reasons */
 
-    return 0;       /* never reached */
+    /* return 0; */ /* never reached */
   }
 
 
