@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003-2007, 2009-2013 by                            */
+/*  Copyright 1996-2000, 2003-2007, 2009-2014 by                            */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -11,7 +11,7 @@
 /*  This is a new version using the MiGS graphics subsystem for             */
 /*  blitting and display.                                                   */
 /*                                                                          */
-/*  Press F1 when running this program to have a list of key-bindings       */
+/*  Press ? when running this program to have a list of key-bindings.       */
 /*                                                                          */
 /****************************************************************************/
 
@@ -99,8 +99,9 @@
     double         radius;
     double         slant;
 
-    int            cff_hinting_engine;
-    int            tt_interpreter_version;
+    unsigned int   cff_hinting_engine;
+    unsigned int   tt_interpreter_version;
+    FT_Bool        warping;
 
     int            font_idx;
     int            offset;            /* as selected by the user */
@@ -115,7 +116,7 @@
   } status = { 1,
                DIM_X, DIM_Y, RENDER_MODE_ALL, FT_ENCODING_NONE,
                72, 48, -1, 1.0, 0.04, 0.04, 0.02, 0.22,
-               0, 0, /* default values are set at runtime */
+               0, 0, 0, /* default values are set at runtime */
                0, 0, 0, 0, 0,
                0, { 0x10, 0x40, 0x70, 0x40, 0x10 }, 2 };
 
@@ -182,7 +183,8 @@
     face = size->face;
     slot = face->glyph;
 
-    radius = status.radius * ( status.ptsize * status.res / 72 );
+    radius = (FT_Fixed)(
+               status.radius * ( status.ptsize * status.res / 72 ) );
 
     FT_Stroker_Set( handle->stroker, radius,
                     FT_STROKER_LINECAP_ROUND,
@@ -193,13 +195,13 @@
 
     for ( i = offset; i < num_indices; i++ )
     {
-      int  glyph_idx;
+      FT_UInt  glyph_idx;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
-        glyph_idx = i;
+        glyph_idx = (FT_UInt)i;
       else
-        glyph_idx = FTDemo_Get_Index( handle, i );
+        glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)i );
 
       error = FT_Load_Glyph( face, glyph_idx,
                              handle->load_flags | FT_LOAD_NO_BITMAP );
@@ -301,13 +303,13 @@
 
     for ( i = offset; i < num_indices; i++ )
     {
-      int  glyph_idx;
+      FT_UInt  glyph_idx;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
-        glyph_idx = i;
+        glyph_idx = (FT_UInt)i;
       else
-        glyph_idx = FTDemo_Get_Index( handle, i );
+        glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)i );
 
       error = FT_Load_Glyph( face, glyph_idx, handle->load_flags );
       if ( !error )
@@ -371,20 +373,20 @@
     slot = face->glyph;
 
     ystr = status.ptsize * status.res / 72;
-    xstr = status.xbold_factor * ystr;
-    ystr = status.ybold_factor * ystr;
+    xstr = (FT_Pos)( status.xbold_factor * ystr );
+    ystr = (FT_Pos)( status.ybold_factor * ystr );
 
     have_topleft = 0;
 
     for ( i = offset; i < num_indices; i++ )
     {
-      int  glyph_idx;
+      FT_UInt  glyph_idx;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
-        glyph_idx = i;
+        glyph_idx = (FT_UInt)i;
       else
-        glyph_idx = FTDemo_Get_Index( handle, i );
+        glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)i );
 
       error = FT_Load_Glyph( face, glyph_idx, handle->load_flags );
       if ( !error )
@@ -482,13 +484,13 @@
 
     for ( i = offset; i < num_indices; i++ )
     {
-      int  glyph_idx;
+      FT_UInt  glyph_idx;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
-        glyph_idx = i;
+        glyph_idx = (FT_UInt)i;
       else
-        glyph_idx = FTDemo_Get_Index( handle, i );
+        glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)i );
 
       error = FTDemo_Draw_Index( handle, display, glyph_idx, &x, &y );
 
@@ -566,7 +568,7 @@
         ch = utf8_next( &p, pEnd );
       }
 
-      glyph_idx = FTDemo_Get_Index( handle, ch );
+      glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)ch );
 
       error = FTDemo_Draw_Index( handle, display, glyph_idx, &x, &y );
 
@@ -681,7 +683,7 @@
       }
 
       start = snprintf( text, 256, "%g: ", pt_size / 64.0 );
-      snprintf( text + start, 256 - start, "%s", p );
+      snprintf( text + start, (unsigned int)( 256 - start ), "%s", p );
 
       p    = text;
       pEnd = p + strlen( text );
@@ -701,7 +703,7 @@
           ch   = utf8_next( &p, pEnd );
         }
 
-        glyph_idx = FTDemo_Get_Index( handle, ch );
+        glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)ch );
 
         error = FTDemo_Draw_Index( handle, display, glyph_idx, &x, &y );
 
@@ -745,16 +747,31 @@
   static void
   event_help( void )
   {
+    char  buf[256];
+    char  version[64];
+
+    const char*  format;
+    FT_Int       major, minor, patch;
+
     grEvent  dummy_event;
 
 
+    FT_Library_Version( handle->library, &major, &minor, &patch );
+
+    format = patch ? "%d.%d.%d" : "%d.%d";
+    sprintf( version, format, major, minor, patch );
+
     FTDemo_Display_Clear( display );
     grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( display->bitmap );
 
-    grWriteln( "FreeType Glyph Viewer - part of the FreeType test suite" );
+    sprintf( buf,
+             "FreeType Glyph Viewer - part of the FreeType %s test suite",
+             version );
+
+    grWriteln( buf );
     grLn();
     grWriteln( "Use the following keys:" );
     grLn();
@@ -790,8 +807,10 @@
     grWriteln( "             engines (if available)                                         " );
     grWriteln( "f           toggle forced auto-         G           show gamma ramp         " );
     grWriteln( "             hinting (if hinting)       g, v        adjust gamma value      " );
+    grWriteln( "w           toggle warping (in light                                        " );
+    grWriteln( "             AA mode, if available)     q, ESC      quit ftview             " );
     grWriteln( "                                                                            " );
-    grWriteln( "a           toggle anti-aliasing        q, ESC      quit ftview             " );
+    grWriteln( "a           toggle anti-aliasing                                            " );
     /*          |----------------------------------|    |----------------------------------| */
     grLn();
     grLn();
@@ -822,7 +841,9 @@
     if ( pitch < 0 )
       pitch = -pitch;
 
-    memset( display->bitmap->buffer, 100, pitch * display->bitmap->rows );
+    memset( display->bitmap->buffer,
+            100,
+            (unsigned int)( pitch * display->bitmap->rows ) );
 
     grWriteCellString( display->bitmap, 0, 0, "Gamma grid",
                        display->fore_color );
@@ -860,7 +881,7 @@
           int     gm  = (int)( 255.0 * pow( p, ggamma ) );
 
 
-          memset( dst, gm, xside * 3 );
+          memset( dst, gm, (unsigned int)( xside * 3 ) );
         }
       }
     }
@@ -871,9 +892,9 @@
 
 
   static int
-  event_cff_hinting_engine_change( int  delta )
+  event_cff_hinting_engine_change( unsigned int  delta )
   {
-    int  new_cff_hinting_engine;
+    unsigned int  new_cff_hinting_engine;
 
 
     if ( delta )
@@ -931,6 +952,34 @@
   }
 
 
+  static int
+  event_warping_change( void )
+  {
+    if ( handle->lcd_mode == LCD_MODE_LIGHT )
+    {
+      FT_Bool  new_warping_state = !status.warping;
+
+
+      error = FT_Property_Set( handle->library,
+                               "autofitter",
+                               "warping",
+                               &new_warping_state );
+
+      if ( !error )
+      {
+        /* Resetting the cache is perhaps a bit harsh, but I'm too  */
+        /* lazy to walk over all loaded fonts to check whether they */
+        /* are auto-hinted, then unloading them explicitly.         */
+        FTC_Manager_Reset( handle->cache_manager );
+        status.warping = new_warping_state;
+        return 1;
+      }
+    }
+
+    return 0;
+  }
+
+
   static void
   event_gamma_change( double  delta )
   {
@@ -1197,6 +1246,10 @@
       }
       break;
 
+    case grKEY( 'w' ):
+      status.update = event_warping_change();
+      break;
+
     case grKeySpace:
       event_render_mode_change( 1 );
       status.update = 1;
@@ -1429,19 +1482,19 @@
          status.render_mode == RENDER_MODE_WATERFALL )
       sprintf( buf, "top left charcode: U+%04X (glyph idx %d)",
                     status.topleft,
-                    FTDemo_Get_Index( handle, status.topleft ) );
+                    FTDemo_Get_Index( handle, (FT_UInt32)status.topleft ) );
     else if ( status.encoding == FT_ENCODING_NONE )
       sprintf( buf, "top left glyph idx: %d",
                     status.topleft );
     else
       sprintf( buf, "top left charcode: 0x%X (glyph idx %d)",
                     status.topleft,
-                    FTDemo_Get_Index( handle, status.topleft ) );
+                    FTDemo_Get_Index( handle, (FT_UInt32)status.topleft ) );
 
     if ( FT_HAS_GLYPH_NAMES( face ) )
     {
-      char*  p;
-      int    format_len, glyph_idx, size;
+      char*         p;
+      unsigned int  format_len, glyph_idx, size;
 
 
       size = strlen( buf );
@@ -1453,11 +1506,11 @@
 
       if ( size >= format_len + 2 )
       {
-        glyph_idx = status.topleft;
+        glyph_idx = (unsigned int)status.topleft;
         if ( status.encoding != FT_ENCODING_NONE         ||
              status.render_mode == RENDER_MODE_TEXT      ||
              status.render_mode == RENDER_MODE_WATERFALL )
-          glyph_idx = FTDemo_Get_Index( handle, status.topleft );
+          glyph_idx = FTDemo_Get_Index( handle, (FT_UInt32)status.topleft );
 
         strcpy( p, format );
         if ( FT_Get_Glyph_Name( face, glyph_idx,
@@ -1705,6 +1758,14 @@
       }
     }
 
+    if ( handle->lcd_mode == LCD_MODE_LIGHT )
+    {
+      sprintf( buf, "warping: %s",
+                    status.warping ? "on" : "off" );
+      grWriteCellString( display->bitmap, 0, (line++) * HEADER_HEIGHT,
+                         buf, display->fore_color );
+    }
+
     line++;
 
     /* embedded bitmaps */
@@ -1882,7 +1943,7 @@
           printf( "\n" );
           exit( 0 );
         }
-        break;
+        /* break; */
 
       case 'w':
         status.width = atoi( optarg );
@@ -1932,6 +1993,9 @@
     FT_Property_Get( handle->library,
                      "truetype",
                      "interpreter-version", &status.tt_interpreter_version );
+    FT_Property_Get( handle->library,
+                     "autofitter",
+                     "warping", &status.warping );
 
     handle->encoding = status.encoding;
 
@@ -1950,7 +2014,7 @@
       Fatal( "could not allocate display surface" );
 
     grSetTitle( display->surface,
-                "FreeType Glyph Viewer - press F1 for help" );
+                "FreeType Glyph Viewer - press ? for help" );
 
     status.num_fails = 0;
 
@@ -2012,7 +2076,7 @@
     FTDemo_Done( handle );
     exit( 0 );      /* for safety reasons */
 
-    return 0;       /* never reached */
+    /* return 0; */ /* never reached */
   }
 
 
